package infomat.algorithms.grouping;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;

import mro.util.*;
import mro.util.properties.*;

import java.util.*;
/*
  import java.util.Vector; 
  import java.util.Map.Entry; 
  import java.util.HashMap; 
  import java.util.HashSet; 
  import java.util.Set; 
*/

/**
 * KMeans
 *      
 * @version          080211
 * @author           Magnus Rosell
 */
public class KMeans extends GroupingAlgorithm implements NeedsISimilarity{ 

    /*
      protected Properties properties;
      protected UniqueIdentifier identifier;
    */

    public static int kmeansClusteringNumber = 1;
	
    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = 
	    new mro.util.properties.Properties("KMeans Properties", CurrentClassGetter.getClassName());
		
	try{
			
	    Property p = new Property("Number of clusters", 5, "Number of clusters");
	    properties.addProperty(p);

	    p = new Property("Init Type", 1, "Kind of initialization");
	    PropertyValueDescription pvd = new PropertyValueDescription("0", "\"Number of clusters\" random texts as start clusters");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("1", "Random clustering as start clusters.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

			
	    p = new Property("Repeat Type", 1, "Iteration criterion");
	    pvd = new PropertyValueDescription("0", "Repeat until there is no change between iterations");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("1", "Repeat \"Repeat number of times\" number of times");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);
			
	    p = new Property("Repeat number of times", 10, "If RepeatType=1");
	    properties.addProperty(p);

	    p = new Property("SimSort", true, "Sort objects in clusters according to similarity to cluster centroid.");
	    pvd = new PropertyValueDescription(true, "Do it.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription(false, "Do not do it.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

			
	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	return properties;
    }

    private ISimilarity similarity;
	
    public KMeans(){
	this.properties = getDefaultProperties();	
    }

    public KMeans(mro.util.properties.Properties properties){
	this.properties=properties;
    }

    public KMeans(ISimilarity similarity){
	this.similarity = similarity;
	this.properties = getDefaultProperties();	
    }

    public KMeans(ISimilarity similarity, 
		  mro.util.properties.Properties properties){
	this.similarity = similarity;
	this.properties=properties;
    }
	
    public void setSimilarity(ISimilarity similarity){
	this.similarity = similarity;
    }

    public ISimilarity getSimilarity(){
	return this.similarity;
    }


    /*
    public IObjectGrouping cluster(IObjectSet ioset){

	IObjectGrouping grouping = cluster((IObjectGroup) ioset);
	String name = grouping.getName();
	int i=1;
	while(!ioset.setGrouping(grouping)){
	    grouping.setName(name+" ("+i+")");
	    i++;
	}

	return grouping;
    }
    */

    public IObjectGrouping cluster(IObjectGroup iogroup){
		
	if(similarity instanceof SparseISimilarity)
	    return cluster(iogroup, (SparseISimilarity) similarity);
	else if(similarity instanceof DenseISimilarity)
	    return cluster(iogroup, (DenseISimilarity) similarity);
	else
	    return null;
    }


    private IObjectGrouping cluster(IObjectGroup iogroup, 
				    SparseISimilarity similarity){
	if(similarity instanceof SparseCartesianSimilarity)
	    return clusterCartesian(iogroup, (SparseCartesianSimilarity) similarity);

	// Init
	Vector<IObjectGroup> groups = init(iogroup);
		
	// Repeat
	int repeatType = (Integer) properties.getPropertyValue("Repeat Type");
	int  repeatNr = (Integer) properties.getPropertyValue("Repeat number of times");

	//	if(repeatType==0){
	boolean repeat=true;
	int iter = 0;
	while(repeat){
	    if(printGroupSizesInIterations){
		printGroupSizes(groups);
	    }
	    Vector<IVector<IVectorIndex>> centroids = 
		new Vector<IVector<IVectorIndex>>();
	    HashMap<IObjectGroup, Vector<IObject>> groupContent = new HashMap<IObjectGroup, Vector<IObject>>();
				
	    for(int g=0; g<groups.size(); g++){
		IObjectGroup group = groups.elementAt(g);

		IVector<IVectorIndex> centroid = 
		    similarity.getGroupRepresentation(group);

		centroids.add(centroid);
		groupContent.put(group, group.getOrdered());
		group.removeAll();
	    }
				
	    for(int o=0; o<iogroup.numberOfIObjects(); o++){
		IObject io = iogroup.getIObjectAt(o);
		IVector<IVectorIndex> representation = 
		    similarity.getRepresentation(io);

		float bestSim=0f;
		int bestC=0;
		for(int c=0; c<centroids.size(); c++){
		    IVector<IVectorIndex> centroid = 
			centroids.elementAt(c);
		    float sim = similarity.sim(representation, centroid);

		    if(sim>bestSim){
			bestSim=sim;
			bestC=c;
		    }
		}
		IObjectGroup group = groups.elementAt(bestC);
		group.add(io);
	    }
		
	    repeat=false;
	    for(int g=0; g<groups.size(); g++){
		IObjectGroup group = groups.elementAt(g);
		if(!group.containsPrecisely(groupContent.get(group))){
		    repeat=true;
		    break;
		}
	    }		

	    iter++;
	    if(repeatType == 1 && iter >= repeatNr)
		repeat=false;
	}

	if(printGroupSizesInIterations){
	    printGroupSizes(groups);
	}

		
	boolean simSort = (Boolean) properties.getPropertyValue("SimSort");
	if(simSort){
	    Sorter.sortInSimOrderWithinClusters(similarity, groups);
	}
		
	String name = "KMeans Clustering "+kmeansClusteringNumber;
	kmeansClusteringNumber++;		
	IObjectGrouping grouping = new IObjectGrouping(name);

	for(int g=0; g<groups.size(); g++){
	    IObjectGroup group = groups.elementAt(g);
	    grouping.add(group);
	    //	    printGroup(group);
	}
		
	return grouping;
    }



    /*************/

    private IObjectGrouping clusterCartesian(IObjectGroup iogroup, 
					     SparseCartesianSimilarity similarity){
	// Init
	Vector<IObjectGroup> groups = init(iogroup);
		
	// Repeat
	int repeatType = (Integer) properties.getPropertyValue("Repeat Type");
	int  repeatNr = (Integer) properties.getPropertyValue("Repeat number of times");

	//	if(repeatType==0){
	boolean repeat=true;
	int iter = 0;
	while(repeat){
	    if(printGroupSizesInIterations){
		printGroupSizes(groups);
	    }
	    Vector<IVector<IVectorIndex>> centroids = 
		new Vector<IVector<IVectorIndex>>();
	    Vector<Double> squaredDistancesFromOrigo = 
		new Vector<Double>();
	    HashMap<IObjectGroup, Vector<IObject>> groupContent = new HashMap<IObjectGroup, Vector<IObject>>();
				
	    for(int g=0; g<groups.size(); g++){
		IObjectGroup group = groups.elementAt(g);

		IVector<IVectorIndex> centroid = 
		    similarity.getGroupRepresentation(group);

		centroids.add(centroid);
		
		squaredDistancesFromOrigo.add(similarity.cartesianSquaredDistanceFromOrigo(centroid));
		groupContent.put(group, group.getOrdered());
		group.removeAll();
	    }
				
	    for(int o=0; o<iogroup.numberOfIObjects(); o++){
		IObject io = iogroup.getIObjectAt(o);
		IVector<IVectorIndex> representation = 
		    similarity.getRepresentation(io);

		float bestSim=0f;
		int bestC=0;
		for(int c=0; c<centroids.size(); c++){
		    IVector<IVectorIndex> centroid = 
			centroids.elementAt(c);
		    double dist = squaredDistancesFromOrigo.get(c);
		    float sim = 
			similarity.sim(representation, centroid, dist);

		    if(sim>bestSim){
			bestSim=sim;
			bestC=c;
		    }
		}
		IObjectGroup group = groups.elementAt(bestC);
		group.add(io);
	    }
		
	    repeat=false;
	    for(int g=0; g<groups.size(); g++){
		IObjectGroup group = groups.elementAt(g);
		if(!group.containsPrecisely(groupContent.get(group))){
		    repeat=true;
		    break;
		}
	    }		

	    iter++;
	    if(repeatType == 1 && iter >= repeatNr)
		repeat=false;
	}

	if(printGroupSizesInIterations){
	    printGroupSizes(groups);
	}

		
	boolean simSort = (Boolean) properties.getPropertyValue("SimSort");
	if(simSort){
	    Sorter.sortInSimOrderWithinClusters(similarity, groups);
	}
		
	String name = "KMeans Clustering "+kmeansClusteringNumber;
	kmeansClusteringNumber++;		
	IObjectGrouping grouping = new IObjectGrouping(name);

	for(int g=0; g<groups.size(); g++){
	    IObjectGroup group = groups.elementAt(g);
	    grouping.add(group);
	    //	    printGroup(group);
	}
		
	return grouping;
    }












    /*
    private IObjectGrouping cluster(IObjectGroup iogroup, 
				    SparseISimilarity similarity){

	// Init
	Vector<IObjectGroup> groups = init(iogroup);
		
	// Repeat
	int repeatType = (Integer) properties.getPropertyValue("Repeat Type");
	int  repeatNr = (Integer) properties.getPropertyValue("Repeat number of times");

	//	if(repeatType==0){
	boolean repeat=true;
	int iter = 0;
	while(repeat){
	    if(printGroupSizesInIterations){
		printGroupSizes(groups);
	    }
	    Vector<HashMap<IObject, Float>> centroids = new Vector<HashMap<IObject, Float>>();
	    HashMap<IObjectGroup, Vector<IObject>> groupContent = new HashMap<IObjectGroup, Vector<IObject>>();
				
	    for(int g=0; g<groups.size(); g++){
		IObjectGroup group = groups.elementAt(g);

		HashMap<IObject, Float> centroid = 
		    similarity.getGroupRepresentation(group);

		centroids.add(centroid);
		groupContent.put(group, group.getOrdered());
		group.removeAll();
	    }
				
	    for(int o=0; o<iogroup.numberOfIObjects(); o++){
		IObject io = iogroup.getIObjectAt(o);
		HashMap<IObject, Float> representation = 
		    similarity.getRepresentation(io);

		float bestSim=0f;
		int bestC=0;
		for(int c=0; c<centroids.size(); c++){
		    HashMap<IObject, Float> centroid = 
			centroids.elementAt(c);
		    float sim = similarity.sim(representation, centroid);

		    if(sim>bestSim){
			bestSim=sim;
			bestC=c;
		    }
		}
		IObjectGroup group = groups.elementAt(bestC);
		group.add(io);
	    }
		
	    repeat=false;
	    for(int g=0; g<groups.size(); g++){
		IObjectGroup group = groups.elementAt(g);
		if(!group.containsPrecisely(groupContent.get(group))){
		    repeat=true;
		    break;
		}
	    }		

	    iter++;
	    if(repeatType == 1 && iter >= repeatNr)
		repeat=false;
	}

		
	boolean simSort = (Boolean) properties.getPropertyValue("SimSort");
	if(simSort){
	    Sorter.sortInSimOrderWithinClusters(similarity, groups);
	}
		
	String name = "KMeans Clustering "+kmeansClusteringNumber;
	kmeansClusteringNumber++;		
	IObjectGrouping grouping = new IObjectGrouping(name);

	for(int g=0; g<groups.size(); g++){
	    IObjectGroup group = groups.elementAt(g);
	    grouping.add(group);
	    //	    printGroup(group);
	}
		
	return grouping;
    }



    */

















    private IObjectGrouping cluster(IObjectGroup iogroup, 
				    DenseISimilarity similarity){

	// Init
	Vector<IObjectGroup> groups = init(iogroup);
		
	// Repeat
	int repeatType = (Integer) properties.getPropertyValue("Repeat Type");
	int  repeatNr = (Integer) properties.getPropertyValue("Repeat number of times");

	//	if(repeatType==0){
	if(printGroupSizesInIterations){
	    printGroupSizes(groups);
	}
	boolean repeat=true;
	int iter = 0;
	while(repeat){
	    Vector<float[]> centroids = new Vector<float[]>();
	    HashMap<IObjectGroup, Vector<IObject>> groupContent = 
		new HashMap<IObjectGroup, Vector<IObject>>();
				
	    for(int g=0; g<groups.size(); g++){
		IObjectGroup group = groups.elementAt(g);

		float[] centroid = 
		    similarity.getGroupRepresentation(group);

		if(centroid.length>0)
		    centroids.add(centroid);
		groupContent.put(group, group.getOrdered());
		group.removeAll();
	    }
				
	    for(int o=0; o<iogroup.numberOfIObjects(); o++){
		IObject io = iogroup.getIObjectAt(o);
		float[] representation = 
		    similarity.getRepresentation(io);

		float bestSim=0f;
		int bestC=0;
		for(int c=0; c<centroids.size(); c++){
		    float[] centroid = 
			centroids.elementAt(c);
		    if(centroid !=null){
			float sim = similarity.sim(representation, centroid);

			if(sim>bestSim){
			    bestSim=sim;
			    bestC=c;
			}
		    }
		}
		IObjectGroup group = groups.elementAt(bestC);
		group.add(io);
	    }
		
	    repeat=false;
	    for(int g=0; g<groups.size(); g++){
		IObjectGroup group = groups.elementAt(g);
		if(!group.containsPrecisely(groupContent.get(group))){
		    repeat=true;
		    break;
		}
	    }		

	    iter++;
	    if(repeatType == 1 && iter >= repeatNr)
		repeat=false;

	    if(printGroupSizesInIterations){
		printGroupSizes(groups);
	    }
	}

		
	boolean simSort = (Boolean) properties.getPropertyValue("SimSort");
	if(simSort){
	    Sorter.sortInSimOrderWithinClusters(similarity, groups);
	}
		
	String name = "KMeans Clustering "+kmeansClusteringNumber;
	kmeansClusteringNumber++;		
	IObjectGrouping grouping = new IObjectGrouping(name);

	for(int g=0; g<groups.size(); g++){
	    IObjectGroup group = groups.elementAt(g);
	    grouping.add(group);
	    //	    printGroup(group);
	}
		
	return grouping;
    }









	
    private Vector<IObjectGroup> init(IObjectGroup iogroup){
	int numOfClusters = (Integer) properties.getPropertyValue("Number of clusters");
	if(numOfClusters > iogroup.numberOfIObjects()){
	    System.out.println("Number of clusters exeeds the number of objects!");
	    System.out.println(" -> initiates with one object per cluster.");
	    Vector<IObjectGroup> groups = new Vector<IObjectGroup>(); 
	    Vector<IObject> ios = iogroup.getOrdered();
	    for(int i=0; i<ios.size(); i++){
		IObjectGroup group = new IObjectGroup("Cluster "+i);
		group.add(ios.elementAt(i));
	    }
	    return groups;
	}
		
	Vector<IObjectGroup> groups = new Vector<IObjectGroup>(); 
	int init = (Integer) properties.getPropertyValue("Init Type");
	if(init==0){
	    HashSet<IObject> ios = new HashSet<IObject>();
	    int c=0;
	    while(c<numOfClusters){
		IObject io = null;
		boolean goOn=true;
		while(goOn){
		    int r = (int) (Math.random()*iogroup.numberOfIObjects());
		    io = iogroup.getIObjectAt(r);
		    //System.out.println("io: "+io.getString());
		    goOn=ios.contains(io);
		}
		ios.add(io);
		IObjectGroup group = new IObjectGroup("Cluster "+c);
		group.add(io);
		groups.add(group);
		c++;
		//System.out.println("c: "+c);
	    }
	}else if(init==1){
	    RandomClusterer rc = new RandomClusterer();
	    try{
		rc.properties.setProperty("Number of clusters", numOfClusters);
	    }catch(PropertyNotFoundException pnfe){
		System.out.println("Property name exception");
		System.exit(0);
	    }
	    IObjectGrouping grouping = rc.cluster(iogroup);
	    groups = grouping.getGroups();
	}
	return groups;	
    }


	
	
	
    public String getInfoString(){
    	return "KMeans";
    }

    public String getAlgorithmName(){
    	return "KMeans";    	
    }
    
    public Vector getPropertyVector(){
    	return properties.getPropertyVector();
    }
    

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }
    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
    }

    public UniqueIdentifier getUniqueIdentifier(){
    	return identifier;
    }

    public void setUniqueIdentifier(UniqueIdentifier uniqueIdentifier){
    	this.identifier=uniqueIdentifier;
    }


    /**
     * Implemented for the "Clustering" course of GSLT 2008.
     * Will not be kept up to date.
     *
     * Runs one iteration of the K-Means algorithm using the specified
     * similarity, starting with the specified IObjectGrouping.
     * Returns a new grouping representing the new state.
     */
    public IObjectGrouping oneIter(IObjectGrouping iogrouping, 
				   SparseISimilarity similarity){
					

	// Init
	Vector<IObjectGroup> groups = iogrouping.getGroups();

	Vector<IVector<IVectorIndex>> centroids = 
	    new Vector<IVector<IVectorIndex>>();
	HashMap<IObjectGroup, Vector<IObject>> groupContent = new HashMap<IObjectGroup, Vector<IObject>>();
				
	for(int g=0; g<groups.size(); g++){
	    IObjectGroup group = groups.elementAt(g);
	    
	    IVector<IVectorIndex> centroid = 
		similarity.getGroupRepresentation(group);
	    
	    centroids.add(centroid);
	    groupContent.put(group, group.getOrdered());
	    group.removeAll();
	}
	
	for(int o=0; o<iogrouping.numberOfIObjects(); o++){
	    IObject io = iogrouping.getIObjectAt(o);
	    IVector<IVectorIndex> representation = 
		similarity.getRepresentation(io);
	    
	    float bestSim=0f;
	    int bestC=0;
	    for(int c=0; c<centroids.size(); c++){
		IVector<IVectorIndex> centroid = 
		    centroids.elementAt(c);
		float sim = similarity.sim(representation, centroid);
		
		if(sim>bestSim){
		    bestSim=sim;
		    bestC=c;
		}
	    }
	    IObjectGroup group = groups.elementAt(bestC);
	    group.add(io);
	}
		
	boolean simSort = (Boolean) properties.getPropertyValue("SimSort");
	if(simSort){
	    Sorter.sortInSimOrderWithinClusters(similarity, groups);
	}
		
	String name = "KMeans Clustering "+kmeansClusteringNumber;
	kmeansClusteringNumber++;		
	IObjectGrouping grouping = new IObjectGrouping(name);

	for(int g=0; g<groups.size(); g++){
	    IObjectGroup group = groups.elementAt(g);
	    grouping.add(group);
	}
		
	return grouping;
    }












}
