package infomat.algorithms.labeling;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;

import mro.util.*;
import mro.util.properties.*;

import java.util.*;


/**
 * Constructs ILabels for IObjectGroups.
 */
public class CentroidBoostLabeler extends Labeler{

    public CentroidBoostLabeler(){
	this.properties = getDefaultProperties();
    }



    // Gr hjlpklass i Properties-paketet 
    // som gr att man kan stt properties frn 
    // Array:er med Object.


    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("CentroidBoostLabeler Properties", CurrentClassGetter.getClassName());
		
	try{


	    Property p = new Property("Number of objects in labels", 10, "The number of objects in each label");
	    properties.addProperty(p);


	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}

	return properties;
    }

    //    public static int numberOfObjects=10;

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }



    /**
     * Can not be used separately, since it relies on having information 
     * on the parent group.
     */
    private ILabel labelGroup(IObjectGroup iog, SparseISimilarity similarity, HashMap<IObject, Float> parentWeightMap){

	/*
	IVector<IObject> vec = similarity.getGroupRepresentation(iog);
	HashMap<IObject, Float> map = vec.getMap();
	*/

	// Temporr fullsning
	HashMap<IObject, Float> map = new HashMap<IObject, Float>();
	IVector<IVectorIndex> vec = 
	    similarity.getGroupRepresentation(iog);
	HashMap<IVectorIndex, Float> indexMap = vec.getMap();
	Iterator<Map.Entry<IVectorIndex, Float>> tmpIt = 
	    indexMap.entrySet().iterator();
	while(tmpIt.hasNext()){
	    Map.Entry<IVectorIndex, Float> entry = tmpIt.next();
	    map.put((IObject) entry.getKey(), entry.getValue());
	}



	HashMap<IObject, Float> weightMap = 
	    new HashMap<IObject, Float>();
	Iterator<Map.Entry<IObject, Float>> it = map.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<IObject, Float> entry = it.next();
	    IObject obj = entry.getKey();
	    float weight = entry.getValue();
	    float labelWeight = (float) 
		(MoreMath.log2(weight) -
		 MoreMath.log2(parentWeightMap.get(obj)));
	    weightMap.put(obj, labelWeight);
	}
	
	// Sortera p labelWeight inom gruppen
	IObjectWeightComparator owc = new IObjectWeightComparator(weightMap);
	TreeMap<IObject, Float> orderedWeightMap = 
	    new TreeMap<IObject, Float>(owc);
	orderedWeightMap.putAll(weightMap);

	ILabel label = new ILabel(iog);
	int numberOfObjects = (Integer) properties.getPropertyValue("Number of objects in labels");
	it = orderedWeightMap.entrySet().iterator();
	for(int i=0; i<numberOfObjects && it.hasNext(); i++){
	    Map.Entry<IObject, Float> entry = it.next();
	    IObject io = entry.getKey();
	    float labelWeight = entry.getValue();
	    label.add(io, labelWeight);
	}

	return label;
    }



    public HashMap<IObjectGroup, ILabel> labelGrouping(IObjectGrouping ioging, SparseISimilarity similarity){

	HierarchicalIObjectGrouping hioging = null;

	if(ioging instanceof HierarchicalIObjectGrouping){
	    hioging = (HierarchicalIObjectGrouping) ioging;

	}else{
	    hioging = new HierarchicalIObjectGrouping("Temporary HierarchicalIObjectGrouping");
	    IObjectGroupNode root = new IObjectGroupNode(ioging, null);
	    Vector<IObjectGroupNode> nodeVec = new Vector<IObjectGroupNode>();
	    nodeVec.add(root);
	    hioging.setRootIObjectGroupNodes(nodeVec);

	    nodeVec = new Vector<IObjectGroupNode>();
	    Vector<IObjectGroup> groups = ioging.getGroups();
	    IObjectGroupNode parent = new IObjectGroupNode(ioging, null);
	    for(int i=0; i<groups.size(); i++){
		IObjectGroup iog = groups.elementAt(i);
		IObjectGroupNode node = new IObjectGroupNode(iog, parent);
		nodeVec.add(node);
	    }
	    hioging.setBottomIObjectGroupNodes(nodeVec);
	}

	HashMap<IObjectGroup, ILabel> groupLabelMap = 
	    new HashMap<IObjectGroup, ILabel>();
	    
	HashMap<IObjectGroupNode, HashMap<IObject, Float>> parentMapMap =
	    new HashMap<IObjectGroupNode, HashMap<IObject, Float>>();

	Vector<IObjectGroupNode> bottomNodes = 
	    hioging.getBottomIObjectGroupNodes();

	for(int i=0; i<bottomNodes.size(); i++){
	    IObjectGroupNode node = bottomNodes.elementAt(i);
	    IObjectGroupNode parent = node.getParent();

	    HashMap<IObject, Float> objWeightMap = parentMapMap.get(parent);
	    if(objWeightMap==null){
		/*
		IVector<IObject> vec =
 		    similarity.getGroupRepresentation(parent.getIObjectGroup());
		objWeightMap = vec.getMap();
		*/

		// Temporr fullsning
		objWeightMap = new HashMap<IObject, Float>();
		IVector<IVectorIndex> vec = 
		    similarity.getGroupRepresentation(parent.getIObjectGroup());
		HashMap<IVectorIndex, Float> indexMap = vec.getMap();
		Iterator<Map.Entry<IVectorIndex, Float>> tmpIt = 
		    indexMap.entrySet().iterator();
		while(tmpIt.hasNext()){
		    Map.Entry<IVectorIndex, Float> entry = tmpIt.next();
		    objWeightMap.put((IObject) entry.getKey(), entry.getValue());
		}




		parentMapMap.put(parent, objWeightMap);
	    }

	    IObjectGroup iog = node.getIObjectGroup();
	    ILabel label = 
		labelGroup(iog, similarity, objWeightMap);
	    groupLabelMap.put(iog, label);
	}

	return groupLabelMap;


	/*
	//	Vector<Map.Entry<IObject, Float>> objWeightVec = 
	//	    ioging.getSortedCentroid(matrix);
	HashMap<IObject, Float> map = 
	similarity.getGroupRepresentation(ioging);
	IObjectFloatMapEntryComparator entryComparator = 
	new IObjectFloatMapEntryComparator(map);
	Vector<Map.Entry<IObject, Float>> objWeightVec = 
	new Vector<Map.Entry<IObject, Float>>(map.entrySet());
	Collections.sort(objWeightVec, entryComparator);

	HashMap<IObject, Float> objWeightOrderMap = 
	new HashMap<IObject, Float>();
	int rank=1;
	for(int i=0; i<objWeightVec.size(); i++){
	IObject io = objWeightVec.elementAt(i).getKey();
	objWeightOrderMap.put(io, (float) rank);
	rank++;
	}


	HashMap<IObjectGroup, ILabel> groupLabelMap = 
	new HashMap<IObjectGroup, ILabel>();

	Vector<IObjectGroup> groups = ioging.getGroups();
	for(int i=0; i<groups.size(); i++){
	IObjectGroup iog = groups.elementAt(i);
	ILabel label = 
	labelGroup(iog, similarity, objWeightOrderMap);
	groupLabelMap.put(iog, label);
	}

	return groupLabelMap;
	*/
    }




}

