package infomat.algorithms.labeling;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;

import mro.util.*;
import mro.util.properties.*;

import java.util.*;


/**
 * Constructs ILabels for IObjectGroups, the first in the centroid.
 */
public class CentroidLabeler extends Labeler{

    public CentroidLabeler(){
	this.properties = getDefaultProperties();
    }

    public CentroidLabeler(mro.util.properties.Properties properties){
	this.properties = properties;
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("Centroid Labeler Properties", CurrentClassGetter.getClassName());
		
	try{


	    Property p = new Property("Number of objects in labels", 10, "The number of objects in each label");
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}

	return properties;
    }

    //    public static int numberOfObjects=10;

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }



    /*********************************/



    /**
     * Constructs ILables for the IObjectGroup by 
     * choosing the higest ranked IObject:s in the centroid.
     * <b>Overrides</b> most of the Labler Properties. 
     * Only looks at "Number of objects in labels".
     */
    public ILabel labelSingleGroup(IObjectGroup iog, SparseISimilarity similarity){

	/*
	IMatrix matrix = similarity.getIMatrix();
	Vector<Map.Entry<IObject, Float>> ios = 
	    iog.getSortedCentroid(matrix);
	*/

	/*
	HashMap<IObject, Float> map = similarity.getGroupRepresentation(iog);
	Vector<Map.Entry<IObject, Float>> ios = 
	    new Vector<Map.Entry<IObject, Float>>(map.entrySet());
	IObjectFloatMapEntryComparator entryComparator = 
	    new IObjectFloatMapEntryComparator(map);
	Collections.sort(ios, entryComparator);
	*/

	Vector<Map.Entry<IObject, Float>> ios = 
	    getSortedGroupRepresentation(iog, similarity);

	ILabel label = new ILabel(iog);
	int numberOfObjects = (Integer) properties.getPropertyValue("Number of objects in labels");
	for(int i=0; i<numberOfObjects && i<ios.size(); i++){
	    Map.Entry<IObject, Float> entry = ios.elementAt(i);
	    label.add(entry.getKey(), entry.getValue());
	}
	return label;
    }

    /**
     * Constructs ILables for all IObjectGroup:s in the IObjectGrouping by
     * choosing the higest ranked IObject:s in the centroids.
     */
    public HashMap<IObjectGroup, ILabel> labelGrouping(IObjectGrouping ioging, SparseISimilarity similarity){
	HashMap<IObjectGroup, ILabel> groupLabelMap = 
	    new HashMap<IObjectGroup, ILabel>();


	//	System.out.println("--- Centroid Labeler");

	Vector<IObjectGroup> groups = ioging.getGroups();
	for(int i=0; i<groups.size(); i++){
	    IObjectGroup iog = groups.elementAt(i);

	    //	    System.out.println("------ group: "+iog.getName());

	    ILabel label = labelSingleGroup(iog, similarity);
	    groupLabelMap.put(iog, label);
	}

	return groupLabelMap;
    }


    /*********************************/
   

}

