package infomat.algorithms.labeling;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;

import mro.util.*;
import mro.util.properties.*;

import java.util.*;
import java.io.*;


/**
 * Constructs ILabels for IObjectGroups.
 */
public abstract class Labeler implements HasProperties{

    mro.util.properties.Properties properties;


    public abstract HashMap<IObjectGroup, ILabel> labelGrouping(IObjectGrouping ioging, SparseISimilarity similarity);


    /*********************************/

    protected Vector<Map.Entry<IObject, Float>> 
	getSortedGroupRepresentation(IObjectGroup group, 
				     SparseISimilarity similarity){

	/*
	IVector<IObject> vec = 
	    similarity.getGroupRepresentation(group);
	HashMap<IObject, Float> map = vec.getMap();

	*/

	// Temporr fullsning
	HashMap<IObject, Float> map = new HashMap<IObject, Float>();
	IVector<IVectorIndex> vec = 
	    similarity.getGroupRepresentation(group);
	HashMap<IVectorIndex, Float> indexMap = vec.getMap();
	Iterator<Map.Entry<IVectorIndex, Float>> it = 
	    indexMap.entrySet().iterator();
	while(it.hasNext()){
	    Map.Entry<IVectorIndex, Float> entry = it.next();
	    map.put((IObject) entry.getKey(), entry.getValue());
	}

	IObjectFloatMapEntryComparator entryComparator = 
	    new IObjectFloatMapEntryComparator(map);
	Vector<Map.Entry<IObject, Float>> objWeightVec = 
	    new Vector<Map.Entry<IObject, Float>>(map.entrySet());


	Collections.sort(objWeightVec, entryComparator);
	return objWeightVec;
    }


    /*********************************/
   

    public static void printLabeling(IObjectGrouping ioging, HashMap<IObjectGroup, ILabel> groupLabelMap){
	System.out.println("************** "+ioging.getName());
	Vector<IObjectGroup> groups = ioging.getGroups();
	for(int i=0; i<groups.size(); i++){
	    IObjectGroup group = groups.elementAt(i);
	    System.out.println("*** "+group.getString());
	    ILabel label = groupLabelMap.get(group);
	    if(label!=null)
		System.out.println(label.getString()+": "+label.getLabelString());
	}      
    }



    private static void printLabel(PrintWriter pw, ILabel label){
	Vector<IObject> ordered = label.getOrdered();
	HashMap<IObject, Float> objectOrderValueMap = 
	    label.getObjectOrderValueMap();
	String labelString="";
	for(int i=0; i<ordered.size(); i++){
	    pw.println(ordered.elementAt(i).getString()+", "+objectOrderValueMap.get(ordered.elementAt(i)));
	}
    }


    public static void printLabeling(PrintWriter pw, IObjectGrouping ioging, HashMap<IObjectGroup, ILabel> groupLabelMap, String info){
	pw.println("Info:\n"+info);
	pw.println("**************** IObjectGrouping: "+ioging.getName());
	Vector<IObjectGroup> groups = ioging.getGroups();
	for(int i=0; i<groups.size(); i++){
	    IObjectGroup group = groups.elementAt(i);
	    pw.println("--- IObjectGroup: "+group.getString());
	    ILabel label = groupLabelMap.get(group);
	    if(label!=null)
		printLabel(pw, label);
	}      
    }

    


}

/**
 * Orders IObjects in order of weight, highest first.
 */
class IObjectWeightComparator implements Comparator{

    private HashMap<IObject, Float> objectOrderValueMap;

    public IObjectWeightComparator(IVector<IObject> vec){
	this.objectOrderValueMap=vec.getMap();
    }
	
    public IObjectWeightComparator(HashMap<IObject, Float> objectOrderValueMap){
	this.objectOrderValueMap=objectOrderValueMap;
    }
	
    public int compare(Object o1, Object o2){
	float f1 = objectOrderValueMap.get((IObject) o1);
	float f2 = objectOrderValueMap.get((IObject) o2);
	if(f1>f2)
	    return -1;
	else if(f2>f1)
	    return 1;
	else{ 
	    return ((IObject) o1).getString().compareTo(((IObject) o2).getString());

	}
    }
}


/**
 * Orders Map.Entry<IObject, Float>:s in order of Float, highest first.
 */
class IObjectFloatMapEntryComparator implements Comparator{
    
    private HashMap<IObject, Float> objectOrderValueMap;
    
    public IObjectFloatMapEntryComparator(IVector<IObject> vec){
	this.objectOrderValueMap=vec.getMap();
    }
	
    public IObjectFloatMapEntryComparator(HashMap<IObject, Float> objectOrderValueMap){
	this.objectOrderValueMap=objectOrderValueMap;
	//	System.out.println("Constructor - map: "+(objectOrderValueMap!=null)+
	//			   ", "+objectOrderValueMap.size());
    }
    
    public int compare(Object o1, Object o2){
	Map.Entry<IObject, Float> e1 = (Map.Entry<IObject, Float>) o1;
	Map.Entry<IObject, Float> e2 = (Map.Entry<IObject, Float>) o2;

	/*
	System.out.println("compare()");
	System.out.println("- o1: "+o1);
	System.out.println("- o2: "+o2);
	System.out.println("- e1: "+e1);
	System.out.println("- e2: "+e2);
	System.out.println("- map: "+(objectOrderValueMap!=null)+
			   ", "+objectOrderValueMap.size());
                   objectOrderValueMap
	float f1 = objectOrderValueMap.get(e1.getValue());
	float f2 = objectOrderValueMap.get(e2.getValue());
	*/

	float f1 = objectOrderValueMap.get(e1.getKey());
	float f2 = objectOrderValueMap.get(e2.getKey());
	if(f1>f2)
	    return -1;
	else if(f2>f1)
	    return 1;
	else{ 
	    return 
		e1.getKey().getString().compareTo(e2.getKey().getString());
	    
	}
    }
}
    

