package infomat.algorithms.labeling;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;

import mro.util.*;
import mro.util.properties.*;

import java.util.*;


/**
 * Constructs ILabels for IObjectGroups.
 */
public class CentroidRankingBoostLabeler{


    /**
     * Can not be used separately, since it relyes on having information 
     * on the parent group.
     */
    private IVector<IObject> labelGroup(IVector<IObject> firstV, 
					IVector<IObject> secondV, 
					SparseISimilarity similarity){
	IVector<IObject> newFV = new IVector<IObject>(firstV);
	IVector<IObject> newSV = new IVector<IObject>(secondV);
	newFV.log(2);
	newSV.log(2);
	
	return newFV.subtract(newSV);
    }



    public HashMap<IObjectGroup, IVector<IObject>> labelGrouping(IObjectGroup ioging, SparseISimilarity similarity){

	HierarchicalIObjectGrouping hioging = null;

	if(ioging instanceof HierarchicalIObjectGrouping){
	    hioging = (HierarchicalIObjectGrouping) ioging;

	}else{
	    hioging = new HierarchicalIObjectGrouping("Temporary HierarchicalIObjectGrouping");
	    IObjectGroupNode root = new IObjectGroupNode(ioging, null);
	    Vector<IObjectGroupNode> nodeVec = new Vector<IObjectGroupNode>();
	    nodeVec.add(root);
	    hioging.setRootIObjectGroupNodes(nodeVec);

	    nodeVec = new Vector<IObjectGroupNode>();
	    Vector<IObjectGroup> groups = ioging.getGroups();
	    IObjectGroupNode parent = new IObjectGroupNode(ioging, null);
	    for(int i=0; i<groups.size(); i++){
		IObjectGroup iog = groups.elementAt(i);
		IObjectGroupNode node = new IObjectGroupNode(iog, parent);
		nodeVec.add(node);
	    }
	    hioging.setBottomIObjectGroupNodes(nodeVec);
	}

	HashMap<IObjectGroup, ILabel> groupLabelMap = 
	    new HashMap<IObjectGroup, ILabel>();
	    
	HashMap<IObjectGroupNode, HashMap<IObject, Float>> parentMapMap =
	    new HashMap<IObjectGroupNode, HashMap<IObject, Float>>();

	Vector<IObjectGroupNode> bottomNodes = 
	    hioging.getBottomIObjectGroupNodes();

	for(int i=0; i<bottomNodes.size(); i++){
	    IObjectGroupNode node = bottomNodes.elementAt(i);
	    IObjectGroupNode parent = node.getParent();

	    HashMap<IObject, Float> objWeightOrderMap = 
		parentMapMap.get(parent);
	    if(objWeightOrderMap==null){
		Vector<Map.Entry<IObject, Float>> objWeightVec = 
		    getSortedGroupRepresentation(parent.getIObjectGroup(), similarity);
		objWeightOrderMap = new HashMap<IObject, Float>();
		int rank=1;
		for(int j=0; j<objWeightVec.size(); j++){
		    IObject io = objWeightVec.elementAt(j).getKey();
		    objWeightOrderMap.put(io, (float) rank);
		    rank++;
		}
		parentMapMap.put(parent, objWeightOrderMap);
	    }

	    IObjectGroup iog = node.getIObjectGroup();
	    ILabel label = 
		labelGroup(iog, similarity, objWeightOrderMap);
	    groupLabelMap.put(iog, label);
	}

	return groupLabelMap;


	/*
	//	Vector<Map.Entry<IObject, Float>> objWeightVec = 
	//	    ioging.getSortedCentroid(matrix);
	HashMap<IObject, Float> map = 
	similarity.getGroupRepresentation(ioging);
	IObjectFloatMapEntryComparator entryComparator = 
	new IObjectFloatMapEntryComparator(map);
	Vector<Map.Entry<IObject, Float>> objWeightVec = 
	new Vector<Map.Entry<IObject, Float>>(map.entrySet());
	Collections.sort(objWeightVec, entryComparator);

	HashMap<IObject, Float> objWeightOrderMap = 
	new HashMap<IObject, Float>();
	int rank=1;
	for(int i=0; i<objWeightVec.size(); i++){
	IObject io = objWeightVec.elementAt(i).getKey();
	objWeightOrderMap.put(io, (float) rank);
	rank++;
	}


	HashMap<IObjectGroup, ILabel> groupLabelMap = 
	new HashMap<IObjectGroup, ILabel>();

	Vector<IObjectGroup> groups = ioging.getGroups();
	for(int i=0; i<groups.size(); i++){
	IObjectGroup iog = groups.elementAt(i);
	ILabel label = 
	labelGroup(iog, similarity, objWeightOrderMap);
	groupLabelMap.put(iog, label);
	}

	return groupLabelMap;
	*/
    }




}

