package infomat.bridges.jsdm;

import infomat.vectorspace.*;

import mro.util.*;
import mro.util.properties.*;

import moj.ri.*;


import java.io.*;
import java.util.*; 
import java.util.zip.*;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * 
 * @version          080608
 * @author           Magnus Rosell
 */
public class JSDMWrapper implements HasProperties{
    mro.util.properties.Properties properties;

    private SparseDistributedMemory sdm;
    private int numberOfLabels;

    //    private HashMap<IObject, RandomLabel> iorlMap;
    private HashMap<IObject, float[]> ioReprMap;

    private IObjectGroup wrappedGroup = null;
    private int numberOfIOsNotInSDM;

    private IObjectGroup fullGroup = null;

    public JSDMWrapper(){ 
	this.properties = getDefaultProperties();
	
	//	int dimensionality, int randomDegree, int seed, int leftWindowSize, int rightWindowSize, WeightingScheme weightingScheme);

	/*
	int dimensionality = 1800;
	int randomDegree = 8;
	int seed = 710225;
	int leftWindowSize =3;
	int rightWindowSize = 3;
	moj.ri.weighting.WeightingScheme weightingScheme = 
	    new moj.ri.weighting.MangesWS();

	sdm = new SparseDistributedMemory(dimensionality, randomDegree, seed, leftWindowSize, rightWindowSize, weightingScheme);
	*/

	sdm = new SparseDistributedMemory();

    }


    public JSDMWrapper(mro.util.properties.Properties properties){
	this.properties = properties;
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("Java Sparse Distributed Memory Properties", CurrentClassGetter.getClassName());
		
	try{

	    // Local row weight
	    Property p = new Property("Least Term Frequency", 0, "Minimum Term Frequency to be Imported");
	    properties.addProperty(p);
	    p = new Property("Least Document Frequency", 0, "Minimum Document Frequency to be Imported");
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}

	return properties;
    }

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }



    /**
     * Loads.
     */
    public void load(String fileName) {
	System.out.println("************** loading "+fileName);

	numberOfLabels = sdm.load(fileName);
	wrap(null);
    }

    /**
     * Loads.
     */
    public void load(String fileName, IObjectGroup group) {
	System.out.println("************** loading "+fileName);

	numberOfLabels = sdm.load(fileName);
	wrap(group);
    }

    private void wrap(IObjectGroup iog){
	Set<Map.Entry<String, RandomLabel>> entries = sdm.entrySet();
	//	iorlMap = new HashMap<IObject, RandomLabel>();
	ioReprMap = new HashMap<IObject, float[]>();
	if(iog==null){
	    fullGroup = 
		new IObjectGroup("RI Group "+System.currentTimeMillis());
	    
	    int id = 0;
	    Iterator<Map.Entry<String, RandomLabel>> it = entries.iterator();
	    while(it.hasNext()){
		Map.Entry<String, RandomLabel> entry = it.next();
		IObject io = new IObject(entry.getKey(), id);
		fullGroup.add(io);
		//		iorlMap.put(io, entry.getValue());
		float[] repr = entry.getValue().getContext();
		normalize(repr);
		if(repr!=null && repr.length>0)
		    ioReprMap.put(io, repr);
		id++;
	    }
	}else{
	    wrappedGroup = 
		new IObjectGroup("RI Wrapped Group "+System.currentTimeMillis());

	    numberOfIOsNotInSDM=0;
	    Vector<IObject> iovec = iog.getOrdered();

	    HashSet<String> stringSet = new HashSet<String>();
	    int maxID = 0;
	    for(int i=0; i<iovec.size(); i++){
		IObject io = iovec.elementAt(i);
		if(io.getID()> maxID)
		    maxID = io.getID();
		RandomLabel rl = sdm.getRandomLabel(io.getString());
		if(rl!=null){
		    //		    iorlMap.put(io, rl);
		    float[] repr = rl.getContext();
		    if(repr!=null && repr.length>0){
			normalize(repr);
			ioReprMap.put(io, repr);
			wrappedGroup.add(io);
			stringSet.add(io.getString());
		    }else{
			numberOfIOsNotInSDM++;
		    }
		}else{
		    numberOfIOsNotInSDM++;
		}
	    }

	    fullGroup = 
		new IObjectGroup("RI Group "+System.currentTimeMillis());
	    fullGroup.addAll(wrappedGroup.getIVector());

	    int id = maxID+1;
	    Iterator<Map.Entry<String, RandomLabel>> it = entries.iterator();
	    while(it.hasNext()){
		Map.Entry<String, RandomLabel> entry = it.next();
		if(!stringSet.contains(entry.getKey())){
		    IObject io = new IObject(entry.getKey(), id);
		    fullGroup.add(io);
		    //		iorlMap.put(io, entry.getValue());
		    float[] repr = entry.getValue().getContext();
		    normalize(repr);
		    if(repr!=null && repr.length>0)
			ioReprMap.put(io, repr);
		    id++;
		}
	    }
	    


	}

    }

    private void normalize(float[] repr){
	//	Iterator<float[]> it = ioReprMap.values().iterator();
	double sum = 0.0f;
	for(int i=0; i<repr.length; i++)
	    sum += Math.pow(repr[i], 2);
	for(int i=0; i<repr.length; i++)
	    repr[i] = (float) Math.sqrt(Math.pow(repr[i],2)/sum);
    }


    public float[] getWrappedRepresentation(IObject io){
	/*
	RandomLabel rl = iorlMap.get(io);
	return rl.getContext();
	*/
	return ioReprMap.get(io);
    }
    
    public RandomLabel getRandomLabel(String word){
	RandomLabel rl = sdm.getRandomLabel(word);
	if(rl.getDimensionality()==0)
	    return rl;
	return rl;
    }

    /*
    public int getDocumentFrequency(String word){
	RandomLabel rl = getRandomLabel();
	if(rl.getDimensionality()==0)
	    return rl;
	return rl;
    }
    */


    //    public IObjectGroup getWholeIObjectGroup(){
    public IObjectGroup getWrappedIObjectGroup(){
	return wrappedGroup;
    }

    public IObjectGroup getFullIObjectGroup(){
	return fullGroup;
    }

}
