package infomat.io.textresult;

import mro.util.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import infomat.vectorspace.*;
import infomat.vectorspace.ordering.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;

import java.io.*;
import java.util.*; 
import java.util.zip.*;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * For loading and saving groupings to text.
 * 
 * @version          080624
 * @author           Magnus Rosell
 */
public class IVectorTextResult{

    public static void printPartIVectorDTD(PrintWriter out) {
	out.println("<!-- IVector -->");
	out.println("<!ELEMENT IVector (IVecEl+)>");
	out.println("<!ELEMENT IVecEl (IVStr, IVVal, IVLoc?)> <!-- An IVector Element -->");
	out.println("<!ELEMENT IVStr (#PCDATA)> <!-- A string representing the IVector Index -->");
	out.println("<!ELEMENT IVVal (#PCDATA)> <!-- Order value -->");
	out.println("<!ELEMENT IVLoc (#PCDATA)> <!-- The location of the actual object -->");
    }

    public static void printPartDescriptionIVectorXSL(PrintWriter pw, String context) throws IOException{

	pw.println("<xsl:template match=\""+context+"DescriptionIObjects/IVector\">");

	pw.println("<b>descr:</b>&#160;");
	pw.println("<xsl:for-each select=\"IVecEl\">");

	pw.println("<xsl:element name=\"a\">");
	pw.println("<xsl:attribute name=\"href\">");
	pw.print("file:///");
	pw.println("<xsl:value-of select=\"IVLoc\"/>");
	pw.println("</xsl:attribute>");

	pw.println("<xsl:attribute name=\"style\">");
	pw.println("text-decoration: none; color: black");
	pw.println("</xsl:attribute>");

	pw.println("<xsl:value-of select=\"IVStr\"/>");
	pw.println("</xsl:element>");
	pw.println("&#160;<xsl:value-of select=\"IVVal\"/>,&#160;");

	pw.println("</xsl:for-each>");
	pw.println("<br />");

	pw.println("</xsl:template>");
	pw.println("");

    }

    public static void printPartRepresentationIVectorXSL(PrintWriter pw, String context) throws IOException{

	pw.println("<xsl:template match=\""+context+"RepresentationIObjects/IVector\">");
	pw.println("<b>repr:</b>&#160;");
	pw.println("<xsl:for-each select=\"IVecEl\">");

	pw.println("<xsl:element name=\"a\">");
	pw.println("<xsl:attribute name=\"href\">");
	pw.print("file:///");
	pw.println("<xsl:value-of select=\"IVLoc\"/>");
	pw.println("</xsl:attribute>");

	pw.println("<xsl:attribute name=\"style\">");
	pw.println("text-decoration: none; color: black");
	pw.println("</xsl:attribute>");

	pw.println("<xsl:value-of select=\"IVStr\"/>");
	pw.println("</xsl:element>");
	pw.println("&#160;<xsl:value-of select=\"IVVal\"/>,&#160;");

	pw.println("</xsl:for-each>");
	pw.println("<br />");

	pw.println("</xsl:template>");
	pw.println("");
    }

    public static void printPartIObjectGroupIVectorXSL(PrintWriter pw, String context) throws IOException{

	pw.println("<xsl:template match=\""+context+"IObjects/IVector\">");
	pw.println("<ol>");	
	pw.println("<xsl:for-each select=\"IVecEl\">");
	pw.println("<li>");	
	pw.println("<xsl:element name=\"a\">");
	pw.println("<xsl:attribute name=\"href\">");
	pw.print("file:///");
	pw.println("<xsl:value-of select=\"IVLoc\"/>");
	pw.println("</xsl:attribute>");
	pw.println("<xsl:value-of select=\"IVStr\"/>");
	pw.println("</xsl:element>");
	pw.println("&#160; (<xsl:value-of select=\"IVVal\"/>)");
	pw.println("</li>");	
	pw.println("</xsl:for-each>");
	pw.println("</ol>");	
	pw.println("</xsl:template>");
	pw.println("");
    }


    /*
    public static void save(File file, IVector<IObject> vec, int numberOfObjects, String unknownLocation){
	try {
	    PrintWriter out =
		new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(file))));
	    save(out, vec, numberOfObjects, unknownLocation);
	    out.close();
	}
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
    }
    */

    public static <T extends IVectorIndex> void saveIVector(PrintWriter out, IVector<T> vec, int numberOfObjects, String unknownLocation){
	out.println("<IVector>");
	Vector<T> ios = vec.getOrderedVector();
	for(int i=0; (i<ios.size() && i<numberOfObjects); i++){
	    T io = ios.elementAt(i);
	    float value = vec.getValue(io);
	    out.println("<IVecEl>");
	    out.println("<IVStr>"+io.getString()+"</IVStr>");
	    if(io.getLocation()==null){
		out.println("<IVLoc>"+unknownLocation+"</IVLoc>");
	    }else{
		//		out.println("<IVLoc>"+io.getLocation()+"</IVLoc>");
		String location = io.getLocation();
		try{
		    File f = new File(io.getLocation());
		    location = f.getCanonicalPath();
		}catch(IOException ioe){
		    // Use io.getLocation()
		}
		out.println("<IVLoc>"+location+"</IVLoc>");
	    }
	    out.println("<IVVal>"+value+"</IVVal>");
	    out.println("</IVecEl>");
	}
	out.println("</IVector>");
    }
}
