package infomat.vectorspace;

import mro.util.*;
import mro.util.properties.*;

import java.lang.Math.*;
import java.util.*;

/**
 * Removes IMatrixCells, rows, and columns depending on their frequency.
 *
 * @version          080218
 * @author           Magnus Rosell
 */
public class IMatrixCellFilter implements HasProperties{
    mro.util.properties.Properties properties;

    public IMatrixCellFilter(){
	this.properties = getDefaultProperties();
    }

    public IMatrixCellFilter(mro.util.properties.Properties properties){
	this.properties = properties;
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("IMatrixCell Filter Properties", CurrentClassGetter.getClassName());
		
	try{

	    Property p = new Property("Remove cells with lesser frequency than", 1, "Cells with low frequency are removed");
	    properties.addProperty(p);

	    /*
	    p = new Property("Remove cells with lesser relative frequency in their row than", 0.0, "Cells with low relative frequency in their rows are removed");
	    properties.addProperty(p);

	    p = new Property("Remove cells with lesser relative frequency in their column than", 0.0, "Cells with low relative frequency in their columns are removed");
	    properties.addProperty(p);
	    */

	    p = new Property("Remove rows with fewer elements than", 2, "Rows with less elements are removed");
	    properties.addProperty(p);
	    
	    p = new Property("Remove rows with higher proportion than", 0.5, "Rows with higher proportion of possible elements are removed.");

	    /*
	    p = new Property("Remove rows with more elements than", -1, "Rows with more elements are removed");
	    PropertyValueDescription pvd = new PropertyValueDescription("-1", "The total number of columns.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("-2", "Half the total number of columns.");
	    p.addValueDescription(pvd);
	    */

	    properties.addProperty(p);
	    	    
	    p = new Property("Remove columns with fewer elements than", 2, "columns with less elements are removed");
	    properties.addProperty(p);

	    p = new Property("Remove columns with higher proportion than", 0.5, "Columns with higher proportion of possible elements are removed.");

	    /*
	    p = new Property("Remove low row-frequency columns", 0.0, "Remove the columns that appear in the fewest rows (the bottom part)");

	    p = new Property("Remove high row-frequency columns", 1.0, "Remove the columns that appear in most rows (the top part)");
	    */


	    /*
	    p = new Property("Remove columns with more elements than", -2, "Columns with more elements are removed");
	    pvd = new PropertyValueDescription("-1", "The total number of rows.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("-2", "Half the total number of rows.");
	    p.addValueDescription(pvd);
	    */
	    properties.addProperty(p);
	    
	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}

	return properties;

	/*
<pgn>Document Properties</pgn>
<p>
<n>Minimum frequency of Words in Document</n><v>0.0</v>
<d>A Word has to be more frequent than this in a document to be stored.</d>
</p>
<p>
<n>Lowest occourrence of Word in Document</n><v>2</v>
<d>A word has to appear more times than this in a document to be saved</d>
</p>
<p>
<n>Minimum Number of Words in Document</n><v>0</v>
<d>A document has to have more Words than this to be stored.</d>
</p>
<p>
<n>Minimum Number of Documents</n><v>3</v>
<d>A Word has to appear in at least this number of Documents.</d>
</p>
<p>
<n>Maximum Number of Documents</n><v>-1</v>
<d>A Word is not stored if it appears in more than this number of Documents. Default (-1) is half the total number of Documents.</d>
</p>
	*/




    }
    

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }


    private void info(IMatrix matrix, IObjectSet rowSet, IObjectSet colSet){
	System.out.println("---");
	System.out.println("Info Matrix: "+matrix.getSize());
	System.out.println("Info rowSet: "+rowSet.numberOfIObjects());
	System.out.println("Info colSet: "+colSet.numberOfIObjects());
    }


    public void filter(IMatrix matrix){

	IObjectSet rowSet = matrix.getRowIObjectSet();
	IObjectSet colSet = matrix.getColumnIObjectSet();

	//	info(matrix, rowSet, colSet);

	System.out.println("Filtering...");
	System.out.println("   before: "+matrix.getNumberOfRowsColumnsCellsString());


	/*
	IObjectGrouping rowGrouping = matrix.getCurrentRowGrouping();
	IObjectGrouping colGrouping = matrix.getCurrentColumnGrouping();
	*/

	// Cells
	int minF = (Integer) properties.getPropertyValue("Remove cells with lesser frequency than");
	Vector cellVec = matrix.getAllCells();
	Vector<IMatrixCell> removedCells = new Vector<IMatrixCell>();
	for(int i=0; i<cellVec.size(); i++){
	    IMatrixCell mc = (IMatrixCell) cellVec.elementAt(i);
	    if(mc.getLocalCount()<minF){
		matrix.removeCell(mc);
		removedCells.add(mc);
	    }
	}

	//	info(matrix, rowSet, colSet);

	for(int i=0; i<removedCells.size(); i++){
	    IMatrixCell mc = removedCells.elementAt(i);
	    matrix.removeCell(mc);

	    if(matrix.getRowIMatrixCellsForColumnObject(mc.getColumnIObject()).size()==0)
		colSet.remove(mc.getColumnIObject());

	    if(matrix.getColumnIMatrixCellsForRowObject(mc.getRowIObject()).size()==0)
		rowSet.remove(mc.getRowIObject());
	}
	//	colSet.purgeGroupingsToSet();
	//	rowSet.purgeGroupingsToSet();
	//	matrix.purgeMatrixToCurrentGroupings();
	matrix.purgeMatrixToGroups(rowSet, colSet);

	//	info(matrix, rowSet, colSet);

		
	/*	
	rowGrouping.remove(mc.getRowIObject());
	colGrouping.remove(mc.getColumnIObject());

	matrix.purgeMatrixToCurrentGroupings();
	*/

	System.out.println("   cells removed, matrix: "+matrix.getNumberOfRowsColumnsCellsString());


	// Rows
	int colSize = colSet.numberOfIObjects();
	int minCols = (Integer) properties.getPropertyValue("Remove rows with fewer elements than");
	//	int maxCols = (Integer) properties.getPropertyValue("Remove rows with more elements than");
	double val = (Double) properties.getPropertyValue("Remove rows with higher proportion than");
	int maxCols = (int) Math.round(val*colSize);
	removeColsOrRows(false, colSize, minCols, maxCols, rowSet, matrix);
	//	info(matrix, rowSet, colSet);

	System.out.println("   rows removed, matrix: "+matrix.getNumberOfRowsColumnsCellsString());


	// Columns
	int rowSize = rowSet.numberOfIObjects();
	int minRows = (Integer) properties.getPropertyValue("Remove columns with fewer elements than");
	//	int maxRows = (Integer) properties.getPropertyValue("Remove columns with more elements than");
	val = (Double) properties.getPropertyValue("Remove columns with higher proportion than");
	int maxRows = (int) Math.round(val*rowSize);
	removeColsOrRows(true, rowSize, minRows, maxRows, colSet, matrix);
	//	info(matrix, rowSet, colSet);


	System.out.println("   columns removed, matrix: "+matrix.getNumberOfRowsColumnsCellsString());


	// Remove rows with few elements.
	colSize = colSet.numberOfIObjects();
	maxCols = -1;
	removeColsOrRows(false, colSize, minCols, maxCols, rowSet, matrix);
	//	info(matrix, rowSet, colSet);


	System.out.println("   rows removed, matrix: "+matrix.getNumberOfRowsColumnsCellsString());

    }

    /**
     * Helper method.
     */
    private void removeColsOrRows(boolean cols, int size, int min, int max, IObjectSet set, IMatrix matrix){
	if(max==-1)
	    max=size;
	else if(max==-2)
	    max=size/2;
	Vector<IObject> removeObjects = new Vector<IObject>();
	Vector<IObject> objects = set.getOrdered();
	for(int i=0; i<objects.size(); i++){
	    IObject obj = objects.elementAt(i);
	    HashSet objSet = null;
	    if(cols)
		objSet=matrix.getRowIMatrixCellsForColumnObject(obj);
	    else
		objSet=matrix.getColumnIMatrixCellsForRowObject(obj);
	    
	    if(objSet!=null){
		if(objSet.size()<min || objSet.size()>max)
		    removeObjects.add(obj);
	    }
	}

	for(int i=0; i<removeObjects.size(); i++){
	    set.remove(removeObjects.elementAt(i));

	    HashSet<IMatrixCell> removedCells = null;
	    if(cols){
		removedCells = matrix.getRowIMatrixCellsForColumnObject(removeObjects.elementAt(i));
	    }else{
		removedCells = matrix.getColumnIMatrixCellsForRowObject(removeObjects.elementAt(i));
	    }
	    
	    if(removedCells!=null){
		Iterator<IMatrixCell> it = removedCells.iterator();
		while(it.hasNext()){
		    IMatrixCell mc = it.next();
		    //		for(int i=0; i<removedCells.size(); i++){
		    //		    IMatrixCell mc = removedCells.elementAt(i);
		    matrix.removeCell(mc);
		}
	    }
	}
	//	set.purgeGroupingsToSet();
	//	matrix.purgeMatrixToCurrentGroupings();
	matrix.purgeMatrixToGroups(matrix.getRowIObjectSet(), matrix.getColumnIObjectSet());
    }


    /*
    private static void removeColsOrRows(boolean cols, int size, int min, int max, IObjectGrouping grouping, IMatrix matrix){
	if(max==-1)
	    max=size;
	else if(max==-2)
	    max=size/2;
	Vector<IObject> removeObjects = new Vector<IObject>();
	Vector<IObject> objects = grouping.getOrdered();
	for(int i=0; i<objects.size(); i++){
	    IObject obj = objects.elementAt(i);
	    HashSet objSet = null;
	    if(cols)
		objSet=matrix.getRowIMatrixCellsForColumnObject(obj);
	    else
		objSet=matrix.getColumnIMatrixCellsForRowObject(obj);
	    
	    if(objSet!=null){
		if(objSet.size()<min || objSet.size()>max)
		    removeObjects.add(obj);
	    }
	}
	for(int i=0; i<removeObjects.size(); i++){
	    grouping.remove(removeObjects.elementAt(i));
	}
	grouping.removeEmptyGroups();
	grouping.sort();
	matrix.purgeMatrixToCurrentGroupings();
    }
    */


}
