package infomat.view.algorithms;

import infomat.algorithms.grouping.*;
import infomat.vectorspace.*;
import infomat.view.*;

import javax.swing.*;

import java.awt.event.*;
import java.awt.*;
import java.util.*;


/**
 * A JPanel for choosing which Clustering Algorithm to use. 
 *
 * @see AlgorithmFrame
 *
 * @version          080409
 * @author           Magnus Rosell
 */
public class AlgorithmSelectionPanel extends JPanel
    implements ActionListener{

    private AlgorithmFrame algFrame;
    private JComboBox cBox;


    private String selected;


    private static Vector<String> algNames = null;
    private static HashMap<String, GroupingAlgorithm> nameAlgMap = null;
	
    private static void initAlgs(){
	if(algNames==null){
	    algNames = new Vector<String>();
	    nameAlgMap = new HashMap<String, GroupingAlgorithm>();
	    
	    GroupingAlgorithm alg = new KMeans();
	    nameAlgMap.put(alg.getAlgorithmName(), alg);
	    algNames.add(alg.getAlgorithmName());

	    alg = new BisectingKMeans();
	    nameAlgMap.put(alg.getAlgorithmName(), alg);
	    algNames.add(alg.getAlgorithmName());

	    alg = new RelativeClusterer();
	    nameAlgMap.put(alg.getAlgorithmName(), alg);
	    algNames.add(alg.getAlgorithmName());

	    alg = new AppearanceRelativeClusterer();
	    nameAlgMap.put(alg.getAlgorithmName(), alg);
	    algNames.add(alg.getAlgorithmName());

	    alg = new RandomClusterer();
	    nameAlgMap.put(alg.getAlgorithmName(), alg);
	    algNames.add(alg.getAlgorithmName());

	    alg = new LocationGrouper();
	    nameAlgMap.put(alg.getAlgorithmName(), alg);
	    algNames.add(alg.getAlgorithmName());
	}
    }



    public AlgorithmSelectionPanel(AlgorithmFrame algFrame){
	this.algFrame = algFrame;
	initAlgs();
	selected = algNames.elementAt(0);
	makeComboBox();
    }

    public boolean actionLeadToChange(){
	String newSelected = (String) cBox.getSelectedItem();
	//	System.out.println("newSelected: "+newSelected);
	if(selected != newSelected){
	    selected = newSelected;
	    return true;
	}
	return false;
    }

    public GroupingAlgorithm getSelectedAlgorithm(){
	return nameAlgMap.get(selected);
    }


    public void paintComponent(Graphics g) {
	super.paintComponent(g);
    }



    public void makeComboBox(){
	removeAll();
	cBox = new JComboBox(algNames);
	cBox.setSelectedItem(selected);
	add(cBox);
	//	cBox.addActionListener(algFrame);
	cBox.addActionListener(this);
	revalidate();
	repaint();
    }

    public void actionPerformed(ActionEvent e) {
	if(e.getSource() instanceof JComboBox){ //AlgorithmSelectionPanel){
	    if(actionLeadToChange()){
		GroupingAlgorithm algorithm = getSelectedAlgorithm();
		algFrame.setAlgorithm(algorithm);
	    }
	}


    }


}


