package infomat.view.group;

import infomat.view.*;
import infomat.view.picture.*;
import infomat.view.similarity.*;
import infomat.view.util.*;
import infomat.view.vector.*;
import infomat.vectorspace.*;
import infomat.vectorspace.vector.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;

/**
 * A tool for displaying the text content of a IObjectGroup.
 *
 * @version          080804
 * @author           Magnus Rosell
 */
public class GroupFrame extends JFrame implements ActionListener, Observer{

    private IObjectGroup group;
    private IObjectGrouping grouping;
    private boolean row;
    private InfomatModel model;
    private InfomatView view;

    private IVectorPanel ivectorPanel;

    private JPanel containerPanel;
    private SmallButton apply;
    private SmallButton cancel;
    private GroupInfo groupInfo;

    public static final Dimension GROUPPANEL_PREFERRED_SIZE = 
	new Dimension(310, 600);

    public GroupFrame(IObjectGroup group, IObjectGrouping grouping, 
		      boolean row, InfomatModel model, InfomatView view){
	super("Group Edit");
	this.group=group;
	this.grouping=grouping;
	this.row=row;
	this.model=model;
	this.view=view;

	model.addObserver(this);
	model.getSimilarityManager().addObserver(this);


	Container container = getContentPane();
	container.setLayout(new BoxLayout(container, BoxLayout.Y_AXIS));
	containerPanel = new JPanel();
	containerPanel.setLayout(new BoxLayout(containerPanel, BoxLayout.Y_AXIS));

	container.add(containerPanel);

	container.setLayout(new GridLayout(1,1));
	setPreferredSize(GROUPPANEL_PREFERRED_SIZE);
	setSize(GROUPPANEL_PREFERRED_SIZE);


	apply=new SmallButton("Apply List Order");
	apply.addActionListener(this);
	cancel=new SmallButton("Cancel");
	cancel.addActionListener(this);

	setVisible(true);

	putAll();
	pack();
    }

    public void invert(){
	row = !row;
	putAll();
    }

	
    public void putAll(){
	containerPanel.removeAll();

	groupInfo = new GroupInfo(group, grouping, row, model);
	Dimension ginfoDim = groupInfo.getPreferredSize();
	/*
	groupInfo.setPreferredSize(ginfoDim);
	groupInfo.setMinimumSize(ginfoDim);
	groupInfo.setMaximumSize(ginfoDim);
	*/

	JScrollPane sp = new JScrollPane();
	sp.setViewportView(groupInfo);
	sp.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
	containerPanel.add(sp);

	Dimension spd = groupInfo.getPreferredSize();
	JScrollBar sb = sp.getHorizontalScrollBar();
	Dimension sbDim = sb.getPreferredSize();
	Dimension panelSize = getPreferredSize();

	spd = new Dimension((int) panelSize.getWidth(), (int) (spd.getHeight()+sbDim.getHeight()+2));
	//	spd = new Dimension((int) spd.getWidth(), (int) (spd.getHeight()+sbDim.getHeight()+2));
	sp.setPreferredSize(spd);
	sp.setMinimumSize(spd);
	sp.setMaximumSize(spd);



	//	groupInfo.setMinimumSize(groupInfo.getPreferredSize());
	//	groupInfo.setMaximumSize(groupInfo.getPreferredSize());


	/*
	    GroupingInfo ginfo = new GroupingInfo(grouping, model, rowPanel);
	    //ginfo.setLayout(new BoxLayout(ginfo, BoxLayout.Y_AXIS));
	    Dimension ginfoDim = ginfo.getPreferredSize();
	    ginfo.setPreferredSize(ginfoDim);
	    ginfo.setMinimumSize(ginfoDim);
	    ginfo.setMaximumSize(ginfoDim);


	    JScrollPane sp = new JScrollPane();
	    sp.setViewportView(ginfo);
	    sp.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
	    add(sp);

	    Dimension spd = ginfo.getPreferredSize();
	    JScrollBar sb = sp.getHorizontalScrollBar();
	    Dimension sbDim = sb.getPreferredSize();
	    spd = new Dimension((int) spd.getWidth(), (int) (spd.getHeight()+sbDim.getHeight()+2));
	    sp.setPreferredSize(spd);
	    sp.setMinimumSize(spd);
	    sp.setMaximumSize(spd);
	*/






	/*
	groupInfo.setBorder(new TitledBorder("Info"));
	groupInfo.setAlignmentX(LEFT_ALIGNMENT);

	Dimension d = groupInfo.getPreferredSize();
	JLabel tmpLabel = new JLabel("tmp");
	d = new Dimension((int) d.getWidth(), 
			  (int) (d.getHeight()+20));//tmpLabel.getHeight()));
	groupInfo.setMinimumSize(d);
	groupInfo.setMaximumSize(d);

	groupInfo.setMinimumSize(groupInfo.getPreferredSize());
	groupInfo.setMaximumSize(groupInfo.getPreferredSize());
	*/
	


	JPanel mainPanel = new JPanel();
	mainPanel.setBorder(new TitledBorder("Main"));
	mainPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	mainPanel.add(apply);
	mainPanel.add(cancel);

	containerPanel.add(mainPanel);
	//	mainPanel.setAlignmentX(LEFT_ALIGNMENT);
	mainPanel.setMinimumSize(mainPanel.getPreferredSize());
	mainPanel.setMaximumSize(mainPanel.getPreferredSize());

	JPanel vecPanel = new JPanel();
	vecPanel.setBorder(new TitledBorder("List"));
	vecPanel.setLayout(new GridLayout(1,1));
	IVector<IObject> ivec = group.getIVector();
	ivectorPanel = new IVectorPanel(model, true, group);
	ivectorPanel.setOrderable(true);
	ivectorPanel.setIVector(ivec);

	ivectorPanel.setSimilarity(model.getISimilarity(row));

	vecPanel.add(ivectorPanel);
	//	vecPanel.setAlignmentX(LEFT_ALIGNMENT);
	containerPanel.add(vecPanel);
	containerPanel.add(Box.createVerticalGlue());
	/*
	  vecPanel.setMinimumSize(vecPanel.getPreferredSize());
	  vecPanel.setMaximumSize(vecPanel.getPreferredSize());
	*/


	/*
	  JPanel tmpPanel = new JPanel();
	  tmpPanel.setBorder(new TitledBorder("Current"));
	  tmpPanel.setLayout(new GridLayout(1,1));
	  tmpPanel.add(currentPixelPanel);
	  tmpPanel.setAlignmentX(LEFT_ALIGNMENT);
	  leftPanel.add(tmpPanel);
	  leftPanel.add(Box.createVerticalGlue());

	  containerPanel.add(leftPanel);
	*/


	containerPanel.revalidate();
	repaint();
	//	    pack();
    }
	
    public IObjectGroup getGroup(){
	return group;
    }

    public void actionPerformed(ActionEvent e) {
	if(e.getSource() instanceof JButton){
	    JButton source = (JButton) e.getSource();
	    if(source == apply){

		int size = grouping.numberOfIObjects();

		IVector<IObject> ivec = ivectorPanel.getIVector();
		group.setIVector(ivec);
		grouping.sort();

		ivectorPanel.setIVector(ivec);

		if(size > grouping.numberOfIObjects())
		    view.resetImage();

		view.forceChange();
		view.getOverview().forceChange();
		model.update();
		//		    model.setChanged();
		//		    model.notifyObservers();
	    }else if(source == cancel){
		view.getGroupFrameManager().dispose(this);
	    }
	}
    }


    /** Called by the Observable */
    public void update(Observable o, Object arg) {
	//	ivectorPanel.setSimilarity(model.getISimilarity(row));
	if(o instanceof InfomatModel)
	    putAll();
	else if(o instanceof SimilarityManager)
	    ivectorPanel.setSimilarity(model.getISimilarity(row));
    }






    /***************************************************/
    private class GroupInfo extends JPanel implements ActionListener {

	private IObjectGroup group;
	private IObjectGrouping grouping;
	private boolean row;
	private InfomatModel model;
	private JTextField groupingNameTextField;
		
	public GroupInfo(IObjectGroup group, IObjectGrouping grouping, 
			 boolean row, InfomatModel model){
	    //super(new BorderLayout());
	    super();
	    this.group=group;
	    this.grouping=grouping;
	    this.row=row;
	    this.model=model;

	    //	    setBorder(new TitledBorder("Info"));
	    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

	    

	    putAll();
	}
		
	public void putAll(){
	    removeAll();
	    String rowOrColumn ="columns";
	    if(row)
		rowOrColumn = "rows";
	    //	    if(grouping == model.getIMatrix().getCurrentRowGrouping())
	    //		rowOrColumn = "rows";




	    /*
	    String text = "<html><h2>"+group.getName()+"</h2></html>";
	    JLabel l = new JLabel(text);
	    l.setHorizontalAlignment(SwingConstants.CENTER);
	    add(l);
	    */

	    JPanel labelPart = new JPanel();
	    labelPart.setLayout(new GridLayout(1,1));
	    JLabel l = new JLabel(group.getName());
	    
	    Font curFont = l.getFont();
	    //	    l.setFont(l.getFont().deriveFont(l.getFont().getSize()+5));
	    l.setFont(new Font(curFont.getFontName(), curFont.getStyle(), curFont.getSize()+5));

	    l.setHorizontalAlignment(SwingConstants.LEFT);

	    labelPart.add(l);

	    add(labelPart);


		
	    JPanel part = new JPanel();
	    part.setLayout(new GridLayout(2,1));
	    part.add(new JLabel("Number of "+rowOrColumn+": "+group.numberOfIObjects()));
	    JPanel part2 = new JPanel();
	    part2.setLayout(new BoxLayout(part2, BoxLayout.X_AXIS));
	    part2.add(new JLabel("Change Name:"));
	    groupingNameTextField = new JTextField(20);
	    groupingNameTextField.setText(group.getName());
	    groupingNameTextField.addActionListener(this);
	    part2.add(groupingNameTextField);   		
	    part.add(part2);
	    add(part);
	    //	    part.setAlignmentX(LEFT_ALIGNMENT);

	    /*
	    int height = (int) (3*groupingNameTextField.getPreferredSize().getHeight() +
				l.getPreferredSize().getHeight());
	    //	    int height = (int) (part.getPreferredSize().getHeight() +
	    //				l.getPreferredSize().getHeight());
	    int width = (int) (getPreferredSize().getWidth());
	    //int width = (int) (l.getPreferredSize().getWidth());
	    Dimension d = new Dimension(width, height);  		
	    setPreferredSize(d);
	    setMinimumSize(d);
	    setMaximumSize(d);
	    */

	    revalidate();
	    repaint();
	}

		
	public void actionPerformed(ActionEvent e) {
	    if(e.getSource() instanceof JTextField){
		JTextField t = (JTextField) e.getSource();
		String oldName = group.getName();
		if(t == groupingNameTextField){
		    String newName = t.getText();
		    group.setName(newName);
		    model.update();
		}
		putAll();
	    }
	}







    }






}
