package infomat.view.pixel;

import infomat.view.*;
import infomat.view.picture.*;
import infomat.view.util.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;

/**
 * A tool for displaying IMatrixCells, and IObject:s for 
 * rows and columns.
 *
 * @version          080724
 * @author           Magnus Rosell
 */
public class PixelFrame extends JFrame{

    private int selectionState=0;
    public final static int SELECTION_STATE_SINGLE = 0;
    public final static int SELECTION_STATE_AREA = 1;

    private int selectWhat=0;
    public final static int SELECT_WHAT_CELLS = 0;
    public final static int SELECT_WHAT_ROWS = 1;
    public final static int SELECT_WHAT_COLUMNS = 2;

    private boolean visible=true;
    private Container container;
    private JPanel containerPanel;
	
    private InfomatModel model;
    private InfomatView view;
    //    private DimensionInfoPanel rowPanel;
    //    private DimensionInfoPanel colPanel;
	
    private JPanel leftPanel;
    private JPanel middlePanel;
    private JPanel rightPanel;

    private ControlPanel controlPanel;
    private SelectionPanel selectionPanel;
    private GatherPanel gatherPanel;
    private PixelPanel currentPixelPanel;
    private PixelPanel selectedPixelPanel;
    private PixelPanel gatheredPixelPanel;

    private RemovePanel removePanel;

    private boolean viewSelected = false;
    private boolean viewGathered = false;
    private boolean viewRemove = false;

    public static final Dimension PIXELPANEL_PREFERRED_SIZE = 
	new Dimension(330, 400);

    public PixelFrame(InfomatModel model, InfomatView view){
	super("Pixels");
	this.model=model;
	this.view=view;

	setVisible(visible);
	//	setPreferredSize(PIXELPANEL_PREFERRED_SIZE);
	container = getContentPane();
	container.setLayout(new BoxLayout(container, BoxLayout.X_AXIS));
	containerPanel = new JPanel();

	container.add(containerPanel);
		

	controlPanel = new ControlPanel(this);
	boolean selectable = false;
	boolean fileable = false;
	boolean orderable = false;
	currentPixelPanel = 
	    new PixelPanel(model, view, selectable, fileable, orderable);

	selectionPanel= new SelectionPanel(this);
	selectable = true;
	fileable = true;
	orderable = true;
	selectedPixelPanel = 
	    new PixelPanel(model, view, selectable, fileable, orderable);


	gatherPanel= new GatherPanel(this);
	selectable = true;
	fileable = true;
	orderable = true;
	gatheredPixelPanel = 
	    new PixelPanel(model, view, selectable, fileable, orderable);

	removePanel = new RemovePanel(this);
	/*
	  removePanel.setMinimumSize(removePanel.getPreferredSize());
	  removePanel.setMaximumSize(removePanel.getPreferredSize());
	*/

	putAll();
	pack();
    }
	
    public void putAll(){
	if(visible){
	    containerPanel.removeAll();
			
	    Dimension d = PIXELPANEL_PREFERRED_SIZE;
	    int panels=1;
	    if(viewSelected){
		d = new Dimension((int) (d.getWidth() + PIXELPANEL_PREFERRED_SIZE.getWidth()), (int) PIXELPANEL_PREFERRED_SIZE.getHeight());
		panels++;
	    }
	    if(viewGathered){
		d = new Dimension((int) (d.getWidth() + PIXELPANEL_PREFERRED_SIZE.getWidth()), (int) PIXELPANEL_PREFERRED_SIZE.getHeight());
		panels++;
	    }
	    if(viewRemove && viewGathered){
		d = new Dimension((int) (d.getWidth() + removePanel.getPreferredSize().getWidth()), (int) PIXELPANEL_PREFERRED_SIZE.getHeight());
		panels++;
	    }
	    

	    containerPanel.setLayout(new GridLayout(1,panels));
	    setPreferredSize(d);
	    setSize(d);


	    leftPanel = new JPanel();
	    leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.Y_AXIS));

	    leftPanel.add(controlPanel);
	    controlPanel.setMinimumSize(controlPanel.getPreferredSize());
	    controlPanel.setMaximumSize(controlPanel.getPreferredSize());
	    controlPanel.setAlignmentX(LEFT_ALIGNMENT);

	    JPanel tmpPanel = new JPanel();
	    tmpPanel.setBorder(new TitledBorder("Current"));
	    tmpPanel.setLayout(new GridLayout(1,1));
	    tmpPanel.add(currentPixelPanel);
	    tmpPanel.setAlignmentX(LEFT_ALIGNMENT);
	    leftPanel.add(tmpPanel);
	    leftPanel.add(Box.createVerticalGlue());

	    containerPanel.add(leftPanel);

	    if(viewSelected){
		middlePanel = new JPanel();
		middlePanel.setLayout(new BoxLayout(middlePanel, BoxLayout.Y_AXIS));
		middlePanel.add(selectionPanel);
		selectionPanel.setMinimumSize(selectionPanel.getPreferredSize());
		selectionPanel.setMaximumSize(selectionPanel.getPreferredSize());
		selectionPanel.setAlignmentX(LEFT_ALIGNMENT);

		
		tmpPanel = new JPanel();
		tmpPanel.setBorder(new TitledBorder("Selected"));
		tmpPanel.setLayout(new GridLayout(1,1));
		tmpPanel.add(selectedPixelPanel);
		middlePanel.add(tmpPanel);
		tmpPanel.setAlignmentX(LEFT_ALIGNMENT);
		middlePanel.add(Box.createVerticalGlue());
		
		
		containerPanel.add(middlePanel);
	    }
	    
	    if(viewGathered){
		rightPanel = new JPanel();
		rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.Y_AXIS));


		rightPanel.add(gatherPanel);
		gatherPanel.setMinimumSize(gatherPanel.getPreferredSize());
		gatherPanel.setMaximumSize(gatherPanel.getPreferredSize());
		gatherPanel.setAlignmentX(LEFT_ALIGNMENT);

		
		tmpPanel = new JPanel();
		tmpPanel.setBorder(new TitledBorder("Gathered"));
		tmpPanel.setLayout(new GridLayout(1,1));
		tmpPanel.add(gatheredPixelPanel);
		rightPanel.add(tmpPanel);
		tmpPanel.setAlignmentX(LEFT_ALIGNMENT);
		rightPanel.add(Box.createVerticalGlue());
		
		
		containerPanel.add(rightPanel);
	    }

	    if(viewRemove && viewGathered){
		containerPanel.add(removePanel);
	    }
	    
	    containerPanel.revalidate();
	    repaint();
	    //	    pack();
	}
    }
	
    public void setCurrentInfo(SparsePixel sp){
	if(visible){
	    currentPixelPanel.set(sp);
	}
    }

    public void selectCurrent(SparsePixel sp){
	if(visible && viewSelected){
	    selectedPixelPanel.set(sp);
	}
    }

    public void select(Vector matrixCells){
	if(visible && viewSelected){
	    selectedPixelPanel.set(matrixCells);
	}
    }

    public void setCurrentInfo(IObjectGroup group, boolean row){
	if(visible){
	    currentPixelPanel.set(group, row);
	}
    }


    public void selectCurrent(IObjectGroup group, boolean row){
	if(visible && viewSelected){
	    selectedPixelPanel.set(group, row);
	}
    }


    ////////
    public void setAllSelectedToGathered(){
	if(visible && viewSelected && viewGathered){
	    gatheredPixelPanel.setAllFrom(selectedPixelPanel);
	}
    }

    public void setSelectedCellsToGathered(){
	if(visible && viewSelected && viewGathered){
	    gatheredPixelPanel.setCellsFrom(selectedPixelPanel);
	}
    }
    
    public void setSelectedRowsToGathered(){
	if(visible && viewSelected && viewGathered){
	    gatheredPixelPanel.setRowsFrom(selectedPixelPanel);
	}
    }
    
    public void setSelectedColumnsToGathered(){
	if(visible && viewSelected && viewGathered){
	    gatheredPixelPanel.setColumnsFrom(selectedPixelPanel);
	}
    }

    ///
    public void setSelectedElementsForGatheredRowsAndColumns(){

	if(visible && viewSelected && viewGathered){

	    IObjectGroup rowGroup = new IObjectGroup("tmp rows");
	    IVector<IVectorIndex> rowIVec = gatheredPixelPanel.getRows();
	    Vector<IVectorIndex> rows = rowIVec.getUnorderedVector();
	    for(int i=0; i<rows.size(); i++){
		IVectorIndex ii = rows.get(i);
		rowGroup.add((IObject) ii,  rowIVec.getValue(ii));
	    }

	    IObjectGroup colGroup = new IObjectGroup("tmp cols");
	    IVector<IVectorIndex> colIVec = gatheredPixelPanel.getColumns();
	    Vector<IVectorIndex> cols = colIVec.getUnorderedVector();
	    for(int i=0; i<cols.size(); i++){
		IVectorIndex ii = cols.get(i);
		colGroup.add((IObject) ii,  colIVec.getValue(ii));
	    }

	    IMatrix subMatrix = 
		model.getIMatrix().subMatrix(rowGroup, colGroup);
	    Vector vec = subMatrix.getAllCells();

	    selectedPixelPanel.setElements(vec);

	}	
    }

    public void setSelectedRowsAndColumnsForGatheredElements(){
	if(visible && viewSelected && viewGathered){
	    
	    selectedPixelPanel.set(gatheredPixelPanel.getCells().getUnorderedVector());
	}	
    }


    ////////
    public void addAllSelectedToGathered(){
	if(visible && viewSelected && viewGathered){
	    gatheredPixelPanel.addAllFrom(selectedPixelPanel);
	}
    }

    public void addSelectedCellsToGathered(){
	if(visible && viewSelected && viewGathered){
	    gatheredPixelPanel.addCellsFrom(selectedPixelPanel);
	}
    }
    
    public void addSelectedRowsToGathered(){
	if(visible && viewSelected && viewGathered){
	    gatheredPixelPanel.addRowsFrom(selectedPixelPanel);
	}
    }
    
    public void addSelectedColumnsToGathered(){
	if(visible && viewSelected && viewGathered){
	    gatheredPixelPanel.addColumnsFrom(selectedPixelPanel);
	}
    }

    ///////
    public void putAllSelectedToGathered(){
	if(visible && viewSelected && viewGathered){
	    gatheredPixelPanel.putAllFrom(selectedPixelPanel);
	}
    }

    public void putSelectedCellsToGathered(){
	if(visible && viewSelected && viewGathered){
	    gatheredPixelPanel.putCellsFrom(selectedPixelPanel);
	}
    }
    
    public void putSelectedRowsToGathered(){
	if(visible && viewSelected && viewGathered){
	    gatheredPixelPanel.putRowsFrom(selectedPixelPanel);
	}
    }
    
    public void putSelectedColumnsToGathered(){
	if(visible && viewSelected && viewGathered){
	    gatheredPixelPanel.putColumnsFrom(selectedPixelPanel);
	}
    }

    ////
    public void setSelectedRowsForGatheredColumns(){
	if(visible && viewSelected && viewGathered){
	    selectedPixelPanel.setRowsFromColumns(gatheredPixelPanel);
	}	
    }
	
    public void setSelectedColumnsForGatheredRows(){
	if(visible && viewSelected && viewGathered){
	    selectedPixelPanel.setColumnsFromRows(gatheredPixelPanel);
	}
    }



    public void removeAllGathered(){
	IVector ivec = gatheredPixelPanel.getCells();
	model.getIMatrix().removeCells(ivec);

	ivec = gatheredPixelPanel.getColumns();
	boolean row = false;
	model.getCurrentGrouping(row).remove(ivec);

	ivec = gatheredPixelPanel.getRows();
	row = true;
	model.getCurrentGrouping(row).remove(ivec);

	view.validateIMatrixBorders();
	view.forceChange();
	view.getOverview().forceChange();
	model.update();
    }

    public void removeGatheredCells(){
	IVector ivec = gatheredPixelPanel.getCells();
	model.getIMatrix().removeCells(ivec);

	view.forceChange();
	view.getOverview().forceChange();
	model.update();
    }


    public void removeGatheredColumns(){
	IVector ivec = gatheredPixelPanel.getColumns();
	boolean row = false;
	model.getCurrentGrouping(row).remove(ivec);


	IObjectGrouping grouping = model.getGUIIMatrix().getCurrentColumnGrouping();
	view.getColumnGroupingManager().getColoringForGrouping(grouping).recolor();
	view.validateIMatrixBorders();
	view.forceChange();
	view.getOverview().forceChange();

	model.update();


	/*
	view.validateIMatrixBorders();
	view.forceChange();
	view.getOverview().forceChange();
	model.update();
	*/
    }


    public void removeGatheredRows(){
	IVector ivec = gatheredPixelPanel.getRows();
	boolean row = true;
	model.getCurrentGrouping(row).remove(ivec);

	IObjectGrouping grouping = model.getGUIIMatrix().getCurrentRowGrouping();
	view.getRowGroupingManager().getColoringForGrouping(grouping).recolor();
	view.validateIMatrixBorders();
	view.forceChange();
	view.getOverview().forceChange();

	model.update();


	/*
	view.validateIMatrixBorders();
	view.forceChange();
	view.getOverview().forceChange();
	model.update();
	*/
    }



    public void toggleVisible(){
	visible=!visible;
	super.setVisible(visible);
    }
    
    /*
      public void setVisible(boolean visible){
      super.setVisible(visible);
      this.visible=visible;
      }
    */

    public boolean getVisible(){
	return visible;
    }
	
    public void toggleShowSelected(){
	viewSelected = !viewSelected;
	putAll();
	//	pack();
    }

    public void toggleShowGathered(){
	viewGathered = !viewGathered;
	putAll();
	//	pack();
    }

    public void toggleShowRemove(){
	viewRemove = !viewRemove;
	putAll();
	//	pack();
    }

    public void setSelectionState(int selectionState){
	this.selectionState=selectionState;
    }

    public int getSelectionState(){
	return selectionState;
    }

    public void setSelectWhat(int selectWhat){
	this.selectWhat = selectWhat;
    }

    public int getSelectWhat(){
	return this.selectWhat;
    }

    public void setAllSelectedFromGathered(){
	selectedPixelPanel.setAllFrom(gatheredPixelPanel);
    }

    public void setCellsSelectedFromGathered(){
	selectedPixelPanel.setCellsFrom(gatheredPixelPanel);
    }

    public void setRowsSelectedFromGathered(){
	selectedPixelPanel.setRowsFrom(gatheredPixelPanel);
    }

    public void setColumnsSelectedFromGathered(){
	selectedPixelPanel.setColumnsFrom(gatheredPixelPanel);
    }





    public void sortGatheredToSelected(boolean row){
	sortGatheredToSelected(row, row);
    }

    public void sortGatheredToSelected(boolean gatheredRow, 
				       boolean selectedRow){

	IVector<IObject> selectedVec = selectedPixelPanel.getColumns();
	IVector<IObject> gatheredVec = gatheredPixelPanel.getColumns();
	if(gatheredRow){
	    gatheredVec = gatheredPixelPanel.getRows();
	}
	if(selectedRow){
	    selectedVec = selectedPixelPanel.getRows();
	}

	
	ISimilarity similarity = model.getISimilarity(gatheredRow);

	IVector<IObject> newIVec = new IVector<IObject>();
	if(similarity instanceof SparseISimilarity){
	    SparseISimilarity sim = (SparseISimilarity) similarity; 

	    Vector<IObject> vio = selectedVec.getUnorderedVector();
	    IVector<IVectorIndex> groupRepr;
	    if(gatheredRow == selectedRow){
		IObjectGroup group = new IObjectGroup("tmp");
		for(int i=0; i<vio.size(); i++){
		    IObject io = vio.get(i);
		    group.add(io);
		}
		groupRepr = sim.getGroupRepresentation(group);
	    }else{
		groupRepr = new IVector<IVectorIndex>();
		for(int i=0; i<vio.size(); i++){
		    IObject io = vio.get(i);
		    groupRepr.add(io, selectedVec.getValue(io));
		}
		System.out.println("groupRepr: \n"+groupRepr+"\n---");
	    }

	    vio = gatheredVec.getUnorderedVector();
	    for(int i=0; i<vio.size(); i++){
		IObject io = vio.get(i);
		IVector<IVectorIndex> repr = 
		    sim.getRepresentation(io);
		newIVec.add(io, sim.sim(repr, groupRepr));
	    }
	}
	
	if(newIVec.size() > 0){
	    if(gatheredRow){
		gatheredPixelPanel.setRows(newIVec);
	    }else{
		gatheredPixelPanel.setColumns(newIVec);
	    }
	}
    }

    /*
    public void sortGatheredToSelected(boolean row){
	IVector<IObject> selectedVec = selectedPixelPanel.getColumns();
	IVector<IObject> gatheredVec = gatheredPixelPanel.getColumns();
	if(row){
	    selectedVec = selectedPixelPanel.getRows();
	    gatheredVec = gatheredPixelPanel.getRows();
	}
	ISimilarity similarity = model.getISimilarity(row);

	IVector<IObject> newIVec = new IVector<IObject>();
	if(similarity instanceof SparseISimilarity){
	    SparseISimilarity sim = (SparseISimilarity) similarity; 

	    IObjectGroup group = new IObjectGroup("tmp");
	    Vector<IObject> vio = selectedVec.getUnorderedVector();
	    for(int i=0; i<vio.size(); i++){
		IObject io = vio.get(i);
		group.add(io);
	    }
	    IVector<IVectorIndex> groupRepr = 
		sim.getGroupRepresentation(group);
	    
	    vio = gatheredVec.getUnorderedVector();
	    for(int i=0; i<vio.size(); i++){
		IObject io = vio.get(i);
		IVector<IVectorIndex> repr = 
		    sim.getRepresentation(io);
		newIVec.add(io, sim.sim(repr, groupRepr));
	    }
	}
	
	if(newIVec.size() > 0){
	    if(row){
		gatheredPixelPanel.setRows(newIVec);
	    }else{
		gatheredPixelPanel.setColumns(newIVec);
	    }
	}
    }
    */



    /**
     * Controls what panels are visable.
     */    
    private class ControlPanel extends JPanel implements ActionListener{

	private PixelFrame pf;
	private SmallButton selectedB;
	private SmallButton gatheredB;
	private SmallButton removeB;

	private AlternatingButton selectWhatButton;
	private String[] choices = {"Elements", "Rows", "Columns"};

	public ControlPanel(PixelFrame pf){
	    this.pf=pf;

	    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

	    JPanel sub1 = new JPanel();
	    sub1.setLayout(new FlowLayout(FlowLayout.LEFT));

	    selectedB = new SmallButton("Select");
	    selectedB.addActionListener(this);
	    sub1.add(selectedB);

	    gatheredB = new SmallButton("Gather");
	    gatheredB.addActionListener(this);
	    sub1.add(gatheredB);

	    removeB = new SmallButton("More");
	    removeB.addActionListener(this);
	    sub1.add(removeB);

	    add(sub1);
	    sub1.setAlignmentX(LEFT_ALIGNMENT);


	    JPanel sub2 = new JPanel();
	    sub2.setLayout(new FlowLayout(FlowLayout.LEFT));
	    JLabel l = new JLabel("View: ");
	    sub2.add(l);
	    selectWhatButton = new AlternatingButton(choices);
	    selectWhatButton.addActionListener(this);
	    sub2.add(selectWhatButton);

	    add(sub2);
	    sub2.setAlignmentX(LEFT_ALIGNMENT);



	    setBorder(new TitledBorder("Main"));

	    revalidate();
	    repaint();
	}
	
	public void actionPerformed(ActionEvent e) {
	    if(e.getSource() instanceof JButton){
		JButton source = (JButton) e.getSource();
		if(source == selectWhatButton){
		    selectWhatButton.press();
		    int selectWhat = 0;
		    String string = selectWhatButton.getText();
		    if(string.equals("Elements")){
			selectWhat = PixelFrame.SELECT_WHAT_CELLS;
		    }else if(string.equals("Rows")){
			selectWhat = PixelFrame.SELECT_WHAT_ROWS;
		    }else if(string.equals("Columns")){
			selectWhat = PixelFrame.SELECT_WHAT_COLUMNS;
		    }
		    pf.setSelectWhat(selectWhat);
		}else if(source == selectedB){
		    pf.toggleShowSelected();
		}else if(source == gatheredB){
		    pf.toggleShowGathered();
		}else if(source == removeB){
		    pf.toggleShowRemove();
		}
	    }
	}

    }



    /**
     * Controls how the pixels are selected.
     */    
    private class SelectionPanel extends JPanel implements ActionListener{

	private PixelFrame pf;
	private AlternatingButton selectB;
	//	private SmallButton gatheredB;
	private String[] selectChoices = {"Single", "All in area"};

	public SelectionPanel(PixelFrame pf){
	    this.pf=pf;
	    

	    setBorder(new TitledBorder("Select"));
	    setLayout(new FlowLayout(FlowLayout.LEFT));
	    JLabel l = new JLabel("What to Select: ");
	    //	    p.add(l);
	    add(l);
	    selectB = new AlternatingButton(selectChoices);
	    selectB.addActionListener(this);
	    add(selectB);
	    /*
	    gatheredB = new SmallButton("Gathered");
	    gatheredB.addActionListener(this);
	    add(gatheredB);
	    */

	    revalidate();
	    repaint();
	}
	
	public void actionPerformed(ActionEvent e) {
	    if(e.getSource() instanceof JButton){
		JButton source = (JButton) e.getSource();
		if(source == selectB){
		    selectB.press();
		    int selectionState = 0;
		    String string = selectB.getText();
		    if(string.equals("Single"))
			selectionState = PixelFrame.SELECTION_STATE_SINGLE;
		    else if(string.equals("All in area"))
			selectionState = PixelFrame.SELECTION_STATE_AREA;
		    pf.setSelectionState(selectionState);
		}
		/*
		else if(source == gatheredB){
		    pf.setSelectFromGathered();
		}
		*/
	    }
	}

    }


    /**
     * Controls which pixels are gathered.
     */    
    private class GatherPanel extends JPanel implements ActionListener{

	private PixelFrame pf;
	private SmallButton allB;
	private SmallButton cellB;
	private SmallButton rowB;
	private SmallButton colB;

	private String[] copyMoods = {"Accumulate", "Add", "Set"};
	private AlternatingButton howCopyB;

	public GatherPanel(PixelFrame pf){
	    this.pf=pf;


	    //	    JPanel top = new JPanel();
	    //	    top.setLayout(new BoxLayout(top, BoxLayout.Y_AXIS));
	    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

	    JPanel sub1 = new JPanel();
	    sub1.setLayout(new BoxLayout(sub1, BoxLayout.Y_AXIS));

	    JPanel sub12 = new JPanel();
	    sub12.setLayout(new FlowLayout(FlowLayout.LEFT));
	    JLabel l = new JLabel("How: ");
	    sub12.add(l);
	    howCopyB = new AlternatingButton(copyMoods);
	    howCopyB.addActionListener(this);
	    sub12.add(howCopyB);
	    sub1.add(sub12);
	    sub12.setAlignmentX(LEFT_ALIGNMENT);



	    JPanel sub11 = new JPanel();
	    sub11.setLayout(new FlowLayout(FlowLayout.LEFT));

	    allB = new SmallButton("All");
	    allB.addActionListener(this);
	    sub11.add(allB);

	    cellB = new SmallButton("Elements");
	    cellB.addActionListener(this);
	    sub11.add(cellB);

	    rowB = new SmallButton("Rows");
	    rowB.addActionListener(this);
	    sub11.add(rowB);

	    colB = new SmallButton("Columns");
	    colB.addActionListener(this);
	    sub11.add(colB);

	    sub1.add(sub11);
	    sub11.setAlignmentX(LEFT_ALIGNMENT);

	    add(sub1);
	    sub1.setAlignmentX(LEFT_ALIGNMENT);


	    /*
	      JPanel sub2 = new JPanel();
	      sub2.setLayout(new FlowLayout(FlowLayout.LEFT));
	      rowB = new SmallButton("Rows");
	      rowB.addActionListener(this);
	      sub2.add(rowB);

	      colB = new SmallButton("Columns");
	      colB.addActionListener(this);
	      sub2.add(colB);

	      add(sub2);
	      sub2.setAlignmentX(LEFT_ALIGNMENT);
	    */

	    setBorder(new TitledBorder("Copy Selected"));

	    
	    revalidate();
	    repaint();

	}
	
	public void actionPerformed(ActionEvent e) {
	    if(e.getSource() instanceof JButton){
		JButton source = (JButton) e.getSource();
		String s = howCopyB.getText();
		if(source == allB){
		    if(s.equals("Set"))
			pf.setAllSelectedToGathered();
		    else if(s.equals("Accumulate"))
			pf.addAllSelectedToGathered();
		    else if(s.equals("Add"))
			pf.putAllSelectedToGathered();
		}else if(source == cellB){
		    if(s.equals("Set"))
			pf.setSelectedCellsToGathered();
		    else if(s.equals("Accumulate"))
			pf.addSelectedCellsToGathered();
		    else if(s.equals("Add"))
			pf.putSelectedCellsToGathered();

		}else if(source == rowB){
		    if(s.equals("Set"))
			pf.setSelectedRowsToGathered();
		    else if(s.equals("Accumulate"))
			pf.addSelectedRowsToGathered();
		    else if(s.equals("Add"))
			pf.putSelectedRowsToGathered();

		}else if(source == colB){
		    if(s.equals("Set"))
			pf.setSelectedColumnsToGathered();
		    else if(s.equals("Accumulate"))
			pf.addSelectedColumnsToGathered();
		    else if(s.equals("Add"))
			pf.putSelectedColumnsToGathered();

		}else if(source == howCopyB){
		    howCopyB.press();
		}
	    }
	}

    }

    /**
     * Used to remove what is gathered.
     */    
    private class RemovePanel extends JPanel implements ActionListener{

	private PixelFrame pf;
	private SmallButton allB;
	private SmallButton cellB;
	private SmallButton rowB;
	private SmallButton colB;

	private SmallButton sortRowRowB;
	private SmallButton sortColColB;
	private SmallButton sortRowColB;
	private SmallButton sortColRowB;

	private SmallButton selectAllB;
	private SmallButton selectCellB;
	private SmallButton selectRowB;
	private SmallButton selectColB;

	private SmallButton selectRowForColB;
	private SmallButton selectColForRowB;
	private SmallButton selectElForRowAndColB;
	private SmallButton selectRowAndColForElB;



	public RemovePanel(PixelFrame pf){
	    this.pf=pf;


	    //	    JPanel top = new JPanel();
	    //	    top.setLayout(new BoxLayout(top, BoxLayout.Y_AXIS));
	    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

	    JPanel sub1 = new JPanel();
	    sub1.setLayout(new BoxLayout(sub1, BoxLayout.Y_AXIS));

	    JPanel sub11 = new JPanel();
	    sub11.setLayout(new FlowLayout(FlowLayout.LEFT));
	    allB = new SmallButton("All");
	    allB.setToolTipText("Remove elements from matrix, and rows and columns from groupings");
	    allB.addActionListener(this);
	    sub11.add(allB);

	    cellB = new SmallButton("Elements");
	    cellB.setToolTipText("Remove elements from matrix");
	    cellB.addActionListener(this);
	    sub11.add(cellB);

	    sub1.add(sub11);
	    sub11.setAlignmentX(LEFT_ALIGNMENT);


	    JPanel sub12 = new JPanel();
	    sub12.setLayout(new FlowLayout(FlowLayout.LEFT));
	    rowB = new SmallButton("Rows");
	    rowB.setToolTipText("Remove rows from grouping");
	    rowB.addActionListener(this);
	    sub12.add(rowB);

	    colB = new SmallButton("Columns");
	    colB.setToolTipText("Remove columns from grouping");
	    colB.addActionListener(this);
	    sub12.add(colB);

	    sub1.add(sub12);
	    sub12.setAlignmentX(LEFT_ALIGNMENT);



	    sub1.setBorder(new TitledBorder("Remove Gathered"));
	    sub1.revalidate();
	    sub1.repaint();

	    Dimension d = sub1.getPreferredSize();
	    //	    d = new Dimension((int) (new JLabel("Remove Gathered x")).getPreferredSize().getWidth(), (int) d.getHeight());
	    sub1.setMinimumSize(d);
	    sub1.setMaximumSize(d);
	    /*
	      sub1.setMinimumSize(sub1.getPreferredSize());
	      sub1.setMaximumSize(sub1.getPreferredSize());
	    */
	    add(sub1);
	    sub1.setAlignmentX(LEFT_ALIGNMENT);





	    JPanel sub2 = new JPanel();
	    sub2.setLayout(new BoxLayout(sub2, BoxLayout.Y_AXIS));
	    /*
	    JLabel l = new JLabel("Gathered to Selected");
	    sub2.add(l);
	    */

	    JPanel rowSub = new JPanel();
	    sortRowRowB = new SmallButton("RowRow");
	    sortRowRowB.addActionListener(this);
	    rowSub.add(sortRowRowB);

	    sortRowColB = new SmallButton("RowCol");
	    sortRowColB.addActionListener(this);
	    rowSub.add(sortRowColB);
	    rowSub.setAlignmentX(LEFT_ALIGNMENT);
	    sub2.add(rowSub);

	    JPanel colSub = new JPanel();
	    sortColRowB = new SmallButton("ColRow");
	    sortColRowB.addActionListener(this);
	    colSub.add(sortColRowB);

	    sortColColB = new SmallButton("ColCol");
	    sortColColB.addActionListener(this);
	    colSub.add(sortColColB);
	    colSub.setAlignmentX(LEFT_ALIGNMENT);
	    sub2.add(colSub);

	    TitledBorder titleBorder = 
		new TitledBorder("Sort Gathered to Selected");
	    //	    titleBorder.setTitleJustification(TitledBorder.RIGHT);
	    sub2.setBorder(titleBorder);
	    
	    sub2.revalidate();
	    sub2.repaint();

	    d = sub2.getPreferredSize();
	    //	    d = new Dimension((int) (new JLabel("Remove Gathered x")).getPreferredSize().getWidth(), (int) d.getHeight());
	    //	    d = new Dimension((int) l.getPreferredSize().getWidth(), (int) d.getHeight());
	    sub2.setMinimumSize(d);
	    sub2.setMaximumSize(d);
	    add(sub2);
	    sub2.setAlignmentX(LEFT_ALIGNMENT);




	    JPanel sub3 = new JPanel();
	    sub3.setLayout(new BoxLayout(sub3, BoxLayout.Y_AXIS));

	    JPanel sub31 = new JPanel();
	    sub31.setLayout(new FlowLayout(FlowLayout.LEFT));
	    selectAllB = new SmallButton("All");
	    selectAllB.addActionListener(this);
	    sub31.add(selectAllB);

	    selectCellB = new SmallButton("Elements");
	    selectCellB.addActionListener(this);
	    sub31.add(selectCellB);

	    sub3.add(sub31);
	    sub31.setAlignmentX(LEFT_ALIGNMENT);
	    d = sub31.getPreferredSize();
	    sub31.setMinimumSize(d);
	    sub31.setMaximumSize(d);



	    JPanel sub32 = new JPanel();
	    sub32.setLayout(new FlowLayout(FlowLayout.LEFT));
	    selectRowB = new SmallButton("Rows");
	    selectRowB.addActionListener(this);
	    sub32.add(selectRowB);

	    selectColB = new SmallButton("Columns");
	    selectColB.addActionListener(this);
	    sub32.add(selectColB);

	    sub3.add(sub32);
	    sub32.setAlignmentX(LEFT_ALIGNMENT);
	    d = sub32.getPreferredSize();
	    sub32.setMinimumSize(d);
	    sub32.setMaximumSize(d);


	    JPanel sub34 = new JPanel();
	    sub34.setLayout(new FlowLayout(FlowLayout.LEFT));
	    selectElForRowAndColB = new SmallButton("El for RC");
	    selectElForRowAndColB.addActionListener(this);
	    add(selectElForRowAndColB);
	    selectElForRowAndColB.setAlignmentX(LEFT_ALIGNMENT);
	    sub34.add(selectElForRowAndColB);

	    selectRowAndColForElB = new SmallButton("RC for El");
	    selectRowAndColForElB.addActionListener(this);
	    add(selectRowAndColForElB);
	    selectRowAndColForElB.setAlignmentX(LEFT_ALIGNMENT);
	    sub34.add(selectRowAndColForElB);

	    sub3.add(sub34);
	    sub34.setAlignmentX(LEFT_ALIGNMENT);
	    d = sub34.getPreferredSize();
	    sub34.setMinimumSize(d);
	    sub34.setMaximumSize(d);


	    JPanel sub33 = new JPanel();
	    sub33.setLayout(new FlowLayout(FlowLayout.LEFT));
	    JLabel l = new JLabel("Sim: ");
	    sub33.add(l);
	    selectColForRowB = new SmallButton("C for R");
	    selectColForRowB.addActionListener(this);
	    add(selectColForRowB);
	    selectColForRowB.setAlignmentX(LEFT_ALIGNMENT);
	    sub33.add(selectColForRowB);

	    selectRowForColB = new SmallButton("R for C");
	    selectRowForColB.addActionListener(this);
	    add(selectRowForColB);
	    selectRowForColB.setAlignmentX(LEFT_ALIGNMENT);
	    sub33.add(selectRowForColB);


	    sub3.add(sub33);
	    sub33.setAlignmentX(LEFT_ALIGNMENT);
	    d = sub33.getPreferredSize();
	    sub33.setMinimumSize(d);
	    sub33.setMaximumSize(d);



	    /*

	private SmallButton selectRowForColB;
	private SmallButton selectColForRowB;
	private SmallButton selectElForRowAndColB;
	private SmallButton selectRowAndColForElB;

	    */





	    titleBorder = 
		new TitledBorder("Select Gathered");
	    sub3.setBorder(titleBorder);
	    
	    sub3.revalidate();
	    sub3.repaint();

	    d = sub3.getPreferredSize();
	    sub3.setMinimumSize(d);
	    sub3.setMaximumSize(d);
	    add(sub3);
	    sub3.setAlignmentX(LEFT_ALIGNMENT);





	    
	    revalidate();
	    repaint();

	}
	
	public void actionPerformed(ActionEvent e) {
	    if(e.getSource() instanceof JButton){
		JButton source = (JButton) e.getSource();
		if(source == allB){
		    pf.removeAllGathered();
		}else if(source == cellB){
		    pf.removeGatheredCells();
		}else if(source == rowB){
		    pf.removeGatheredRows();
		}else if(source == colB){
		    pf.removeGatheredColumns();
		}else if(source == sortRowRowB){
		    pf.sortGatheredToSelected(true);
		}else if(source == sortColColB){
		    pf.sortGatheredToSelected(false);
		}else if(source == sortRowColB){
		    pf.sortGatheredToSelected(true, false);
		}else if(source == sortColRowB){
		    pf.sortGatheredToSelected(false, true);
		}else if(source == selectAllB){
		    pf.setAllSelectedFromGathered();
		}else if(source == selectCellB){
		    pf.setCellsSelectedFromGathered();
		}else if(source == selectRowB){
		    pf.setRowsSelectedFromGathered();
		}else if(source == selectColB){
		    pf.setColumnsSelectedFromGathered();

		}else if(source == selectElForRowAndColB){
		    pf.setSelectedElementsForGatheredRowsAndColumns();
		}else if(source == selectRowAndColForElB){
		    pf.setSelectedRowsAndColumnsForGatheredElements();

		}else if(source == selectRowForColB){
		    pf.setSelectedRowsForGatheredColumns();
		}else if(source == selectColForRowB){
		    pf.setSelectedColumnsForGatheredRows();

		}




	    }
	}

    }




}
