package infomat.view.similarity;

import infomat.algorithms.grouping.*;
import infomat.io.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.view.*;
import infomat.view.util.*;
import infomat.view.grouping.settings.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;

import java.awt.*;
import java.awt.Dialog.*;//ModalityType;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import java.io.*;

import javax.swing.*;


	
/**
 *
 * @version          080812
 * @author           Magnus Rosell
 */
public class SimilarityManager extends Observable{

    private InfomatModel model;

    private SparseISimilarity rowSimilarity;
    private SparseISimilarity columnSimilarity;


    public SimilarityManager(InfomatModel model){
	this.model=model;
    }

    public void update(){
	setChanged();
	notifyObservers();
    }

    /**
     * Called from SimilarityFrame.
     */
    protected void invert(){
	SparseISimilarity tmpSimilarity = rowSimilarity;
	rowSimilarity = columnSimilarity;
	columnSimilarity = tmpSimilarity;
    }

    public SparseISimilarity getSimilarity(boolean row){

	SparseISimilarity sim = rowSimilarity;

	if(row){
	    //	    return rowSimilarity;
	}else{
	    sim = columnSimilarity;

	    //	    return columnSimilarity;
	}

	SparseISimilarity retSim = null;
	if(sim instanceof SparseDotSimilarity){
	    retSim = new SparseDotSimilarity((SparseDotSimilarity) sim);
	}
	else if(sim instanceof SparseOverlapSimilarity){
	    retSim = new SparseOverlapSimilarity((SparseOverlapSimilarity) sim);
	}
	else if(sim instanceof SparseCartesianSimilarity){
	    retSim = new SparseCartesianSimilarity((SparseCartesianSimilarity) sim);
	}

	if(retSim instanceof NeedsIMatrix){
	    ((NeedsIMatrix) retSim).setIMatrix(model.getIMatrix());
	}
	if(retSim instanceof NeedsIMatrixDimension){
	    ((NeedsIMatrixDimension) retSim).setIMatrixDimension(row);
	}

	return retSim;
    }


    public void setSimilarity(SparseISimilarity sim, boolean row){
	if(row){
	    this.rowSimilarity=sim;
	}else{
	    this.columnSimilarity=sim;
	}
	update();
    }
 
    
    /*
    public void print(){
	System.out.println("-------- SimilarityManager------------");
	print(true);
	System.out.println("         -----------------            ");
	print(false);
	System.out.println("--------------------------------------");
    }

    public void print(boolean row){
	ISimilarity sim = getSimilarity(row);
	System.out.println("row: "+row);
	if(sim==null){
	    System.out.println("Sim: "+null);
	    return;
	}
	    
	System.out.println("Sim: "+sim.getName());


	if(sim instanceof SparseDotSimilarity){
	    SparseDotSimilarity spSim = (SparseDotSimilarity) sim;
	    System.out.println(" - row: "+spSim.getIMatrixDimension());
	    IMatrix matrix = spSim.getIMatrix();
	    if(matrix==null)
		System.out.println(" - matrix: "+matrix);
	    else
		System.out.println(" - matrix: "+matrix.getSize());
	    
	}
    }
    */

}

