package infomat.view.vector;

import infomat.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.evaluation.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;
import infomat.vectorspace.weighting.*;

import mro.util.*;
import mro.util.properties.*;
import mro.util.measures.*;

import java.io.*;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.xml.sax.SAXException;

import javax.swing.*;
import java.awt.*;


/**
 * Simple class for testing IVectorPanel.
 *
 * <code>javac -deprecation -classpath ./classes/ -sourcepath ./src ./src/infomat/view/vector/Test.java -d classes</code>
 *
 *
 * <br>
 * <code>java -cp .:classes/ infomat.view.vector.Test</code>
 *
 * @version          080703
 * @author           Magnus Rosell
 *
 */
public class Test extends JFrame{
    public static Dimension TEST_PREFERRED_SIZE = 
	new Dimension(300, 500);
    

    private IVectorPanel ivp;

    public Test(IObjectGroup group){
	super("Test");
	Container container = getContentPane();

	if(group!=null)
	    ivp = new IVectorPanel(true, group);
	else
	    ivp = new IVectorPanel(true, true);

	ivp.setOrderable(true);

	/*
	JScrollPane sp = new JScrollPane();
	sp.setViewportView(ivp);
	container.add(sp);
	*/

	container.add(ivp);

	setVisible(true);
	setPreferredSize(TEST_PREFERRED_SIZE);
	pack();
    }

    public void setIVector(IVector<IVectorIndex> vec){
	ivp.setIVector(vec);
    }


    public static void main(String[] args) throws IOException, PropertyNotFoundException{

	// Load the matrix
	// This one has texts as rows and words as columns
	String matrixFileName = "example/matrix.xml";
	File matrixFile = new File(matrixFileName);
	IMatrixIO iomatrix = new IMatrixIO();
	IMatrix matrix = iomatrix.load(matrixFile);

	// Load a categorization
	String categorizationFileName = "example/row_grouping.xml";
	File categorizationFile = new File(categorizationFileName);
	IObjectGroupingIO iobio = new IObjectGroupingIO();
	boolean rows=true; // row categorization
	IObjectGrouping categorization = 
	    iobio.load(rows, matrix, categorizationFile);
	

	// Filter the matrix
	// The example is to small
	/*
	IMatrixCellFilter matrixCellFilter = new IMatrixCellFilter();
	matrixCellFilter.filter(matrix);
	*/

	// Weight the matrix
	TFIDFWeighter weighter = new TFIDFWeighter();
	weighter.weight(matrix);

	// Cluster the rows (texts) of the matrix
	IObjectSet rowSet = matrix.getRowIObjectSet();
	IObjectSet colSet = matrix.getColumnIObjectSet();

	// Similarity
	boolean row = true; // Similarity between the rows of the matrix.
	SparseDotSimilarity dotSim = new SparseDotSimilarity(matrix, row);

	// Cluster
	KMeans kmeans = new KMeans(dotSim);
	IObjectGrouping clustering = kmeans.cluster(rowSet);
	clustering.removeEmptyGroups();

	IObjectGroup cluster = clustering.getGroups().get(0);
	IVector<IVectorIndex> vec = dotSim.getGroupRepresentation(cluster);

	Test t = new Test(colSet);
	t.setIVector(vec);
    }





}
