package mro.util.experimentation;

import mro.util.*;
import mro.util.properties.*;
import mro.util.measures.*;

import java.io.*;
import java.util.HashMap;
import java.util.Vector;
import java.util.Arrays;

import org.xml.sax.SAXException;



/**
 * Class for summing up an experiment.
 *
 * <br>
 * <br>
 * <code>java -cp classes/ mro.util.experimentation.ExperimentResultGenerator</code>
 * <br>
 * <br>
 * <code>javac -deprecation -classpath ./classes/ -sourcepath ./src ./src/mro/util/experimentation/*.java -d classes</code>
 *
 * @version          080512
 * @author           Magnus Rosell
 *
 */
public class ExperimentResultGenerator implements HasProperties{


    private ResultPool resultPool = new ResultPool();
    private PropertiesPool pp = new PropertiesPool();
    private Properties properties = null;
    
    public ExperimentResultGenerator(){
	this.properties = getDefaultProperties();
    }


    public Properties getDefaultProperties(){

	Properties properties = 
	    new Properties("Experiment Result Generator Properties", CurrentClassGetter.getClassName());
		
	try{
			
	    /****************/
	    Property p = new Property("Result Path", null, "Result Path");
	    properties.addProperty(p);

	    p = new Property("Average Path", null, "Average Path");
	    properties.addProperty(p);

	    p = new Property("Result Table Structure File", null, "Result Table Structure File");
	    properties.addProperty(p);

	    p = new Property("Result Latex Table File", null, "Result Table File");
	    properties.addProperty(p);

	    p = new Property("Result Matlab Matrix File", null, "Result Table File");
	    properties.addProperty(p);

	    p = new Property("Result CSV Matrix File", null, "Result Table File");
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	return properties;
    }

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
    }




    public static void main(String[] args) 
	throws org.xml.sax.SAXException, IOException, 
	       PropertyNotFoundException, MeasureNameConflictException{

	System.out.println("main()");
	ExperimentResultGenerator exp = new ExperimentResultGenerator();
	exp.doIt(args);
    }




    public void doIt(String[] args) 
	throws IOException, SAXException, PropertyNotFoundException, MeasureNameConflictException{

	System.out.println("doIt()");
	checkInput(args);

	String resultPathName = (String) getPropertyValue("Result Path");
	String averagePathName = (String) 
	    getPropertyValue("Average Path");
	String resTableStructureFileName = (String) 
	    properties.getProperty("Result Table Structure File").getValue();
	String resTexTableFileName = (String) 
	    properties.getProperty("Result Latex Table File").getValue();
	String resMatlabMatrixFileName = (String) 
	    properties.getProperty("Result Matlab Matrix File").getValue();
	String resCSVMatrixFileName = (String) 
	    properties.getProperty("Result CSV Matrix File").getValue();

	if(resultPathName!=null){
	    // average
	    doOneDirectory(resultPathName);
	    recurseResultPath(new File(resultPathName));
	    writeAverageMeasuresFiles();
	}
	else if(resTableStructureFileName!=null){
	    // table
	    ResultTableStructure rts = 
		ResultTableStructure.load(resTableStructureFileName);
	    String commonPath = rts.getCommonPath();
	    Vector<String> paths = 
		rts.getPathTable().getAllPathsWithCommonPath();
	    for(int i=0; i<paths.size(); i++){
		doOneDirectory(paths.elementAt(i));
	    }

	    ResultTable rt = new ResultTable(rts, resultPool);
	    if(resTexTableFileName!=null)
		rt.constructLatexTableFile(resTexTableFileName);
	    else if(resMatlabMatrixFileName!=null)
		rt.constructMatlabMatrixFile(resMatlabMatrixFileName);
	    else if(resCSVMatrixFileName!=null)
		rt.constructCSVMatrixFile(resCSVMatrixFileName);

	}
    }



    private void recurseResultPath(File path) 
	throws IOException, SAXException, MeasureNameConflictException{
	if(!path.isDirectory())
	    badInput("Specified \"Result Path\" not a path!");

	//	if(!processDir(path.getPath()))
	//	    return;


	System.out.println("resurseResultPath()");

	File[] list = path.listFiles();
	for(int i=0; i<list.length; i++){
	    //	    System.out.println(list[i].getPath());
	    if(list[i].isDirectory()){

		//		if(!processDir(list[i].getPath()))
		//		    continue;

		doOneDirectory(list[i].getCanonicalPath());
		recurseResultPath(list[i]);
	    }
	}
    }


    private void doOneDirectory(String dir) throws IOException, SAXException, MeasureNameConflictException{
	System.out.println("doOneDirectory(): "+dir);

	File fileDir = new File(dir);

	Averagor averagor = new Averagor();
	Measures measures = averagor.readAllMeasuresInDirectory(fileDir);
	PropertiesPool pp = new PropertiesPool();
	pp.loadPropertiesInDirectory(dir);

	resultPool.addResult(dir, measures, pp);
    }


    public void writeAverageMeasuresFiles() throws IOException{

	String resultPathName = 
	    //	    PathHelper.convertToCurrentFileSeparator((String) getPropertyValue("Result Path"));
	    (String) getPropertyValue("Result Path");
	File resultPath = new File(resultPathName);
	resultPathName = resultPath.getCanonicalPath();

	String averagePathName = 
	    (String) getPropertyValue("Average Path");
	File averagePath = new File(averagePathName);

	Vector<Result> resultVector = resultPool.getResultVector();
	for(int i=0; i<resultVector.size(); i++){
	    Result result = resultVector.elementAt(i);
	    Measures ms = result.measures;
	    PropertiesPool pp = result.pp;

	    String currentResultPathName = 
		//		resultPool.getPathForResult(result);
		PathHelper.convertToCurrentFileSeparator(resultPool.getPathForResult(result));


	    //	    System.out.println(" resultPathName "+resultPathName);
	    System.out.println(" *  "+currentResultPathName);


	    String longest = PathHelper.longestCommonStart(currentResultPathName, resultPathName);
	    //	    System.out.println(" longest "+longest);

	    String outResultPathName = averagePath.getCanonicalPath();
	    //	    System.out.println(" outResultPathName "+outResultPathName);

	    String extra = PathHelper.overshoootForSecond(longest, currentResultPathName);
	    //	    System.out.println(" extra "+extra);

	    outResultPathName = outResultPathName + File.separator + extra;

	    /*
	    String outResultPath = null;
	    if(currentResultPath.equals(resultPathName))
		outResultPath = averagePathName + File.separator;
	    else
		outResultPath = averagePathName + currentResultPath.substring(resultPathName.length(), currentResultPath.length());

	    */
	    System.out.println(" -> "+outResultPathName);

	    File path = new File(outResultPathName);
	    if(!path.exists())
		path.mkdirs();

	    if(ms!=null)
		ms.save(outResultPathName+File.separator+"average.xml");
	    if(pp!=null)
		pp.savePropertiesToDirectory(outResultPathName+File.separator);
	}

    }



    private void badInput(String comment){
	System.out.println("Bad input!");
	System.out.println(comment);
	System.out.println("\njava mro.experimentation.ExperimentResultGenerator <params>");
	System.out.println("\n  <params>:");
	System.out.println("      (-resultPath <res-path> -averagePath <avg-path>) |"); 
	System.out.println("      (-structure <table-file.xml> -latex <res-file.tex>) |");
	System.out.println("      (-structure <table-file.xml> -matlab <res-file.m>) |");
	System.out.println("      (-structure <table-file.xml> -csv <res-file.csv>)");

	//	System.out.println("   (-resultTableStructureFile <table-file.xml> -resultMatlabFile <res-file.mat> -measureNames <string1 string2 ...>) |");
	System.out.println("\n");
	System.exit(0);
    }

    
    private void checkInput(String[] args) throws IOException, SAXException, PropertyNotFoundException{

	if(args.length==0)
	    badInput("");//"To few arguments.");

	for(int i=0; i<args.length; i++){
	    if(args[i].equals("-resultPath")){
		setProperty("Result Path", args[i+1]);
	    }
	    else if(args[i].equals("-averagePath")){
		setProperty("Average Path", args[i+1]);
	    }
	    else if(args[i].equals("-structure")){
		setProperty("Result Table Structure File", args[i+1]);
	    }
	    else if(args[i].equals("-latex")){
		setProperty("Result Latex Table File", args[i+1]);
	    }
	    else if(args[i].equals("-matlab")){
		setProperty("Result Matlab Matrix File", args[i+1]);
	    }
	    else if(args[i].equals("-csv")){
		setProperty("Result CSV Matrix File", args[i+1]);
	    }
	}

	String resPath = (String) 
	    properties.getProperty("Result Path").getValue();
	String avgPath = (String) 
	    properties.getProperty("Average Path").getValue();
	String resTableStructureFile = (String) 
	    properties.getProperty("Result Table Structure File").getValue();
	String resTexTableFile = (String) 
	    properties.getProperty("Result Latex Table File").getValue();
	String resMatlabMatrixFile = (String) 
	    properties.getProperty("Result Matlab Matrix File").getValue();
	String resCSVMatrixFile = (String) 
	    properties.getProperty("Result CSV Matrix File").getValue();
	if(resPath!=null && avgPath==null)
	    badInput("");
	if(resPath!=null && resTableStructureFile!=null)
	    badInput("");
	if(resTableStructureFile!=null && 
	   (resMatlabMatrixFile==null && 
	    resTexTableFile==null &&
	    resCSVMatrixFile==null))
	    badInput("");
    }



}
